# !/usr/bin/env python
# -*- coding: utf-8 -*-
# @module: negocio.py
# @author: Gustavo

from presentacion import *
from persistencia import *

''' ######################################################### '''
''' ###############  Paquete Control Entrada  ############### '''
''' ######################################################### '''

from multiprocessing import Process
'''from threading import Thread'''

class ControlModeloEntrada(object):
    '''
    Clase Abstracta para el control del Modelo Entrada
    '''

    def cargarDatos(self, objeto, valor):
        raise NotImplementedError("El método no está implementado en esta clase")
    

class ControlMenuFun(ControlModeloEntrada):
    '''
    Clase para el control del Menu
    '''

    def cargarDatos(self, menu, seleccion):
        ''' Inserta las acciones en el parámetro menú según este o no seleccionado un proyecto. '''

        menu.borrarAcciones()
        consulta="select id, nombre, url, icono, seleccion from acciones where seleccion="+str(seleccion)
        bd=BaseDatosWeb()
        respuesta = bd.sentencia(consulta)
        for indiceControlMenu in range(respuesta.numeroResultados()):
            dato=respuesta.entrada()
            accion=Funcion()
            accion.setDatos(dato)
            menu.insertarAccion(accion)
        bd.desconectar()


class ControlMenuEst(ControlModeloEntrada):
    '''
    Clase para el control del Menu
    '''    
    
    def cargarDatos(self, menu, tipo):
        ''' Inserta la lista de estadisticas del tipo en el parámetro menu. '''

        consulta="select id, nombre, url, icono, tipo from estadisticas where tipo="+str(tipo)
        bd=BaseDatosWeb()
        respuesta = bd.sentencia(consulta)
        for indiceControlMenu in range(respuesta.numeroResultados()):
            dato=respuesta.entrada()
            estadistica=Estadistica()
            estadistica.setDatos(dato)
            menu.insertarAccion(estadistica)
        bd.desconectar()


class ControlFuncion(ControlModeloEntrada):
    '''
    Clase para el control del Menu Estadistica
    '''

    def cargarDatos(self, id, funcion):
        ''' Carga los datos de la estadistica con id en el parametro estadistica. '''

        consulta="select id, nombre, url, icono, seleccion from acciones where id="+str(id)
        bd=BaseDatosWeb()
        respuesta = bd.sentencia(consulta)
        if respuesta.numeroResultados()!=0:
            dato=respuesta.entrada()
            funcion.setDatos(dato)
        bd.desconectar()


class ControlEstadistica(ControlModeloEntrada):
    '''
    Clase para el control del Menu Estadistica
    '''

    def cargarDatos(self, id, estadistica):
        ''' Carga los datos de la estadistica con id en el parametro estadistica. '''

        consulta="select id, nombre, url, icono, tipo from estadisticas where id="+str(id)
        bd=BaseDatosWeb()
        respuesta = bd.sentencia(consulta)
        if respuesta.numeroResultados()!=0:
            dato=respuesta.entrada()
            estadistica.setDatos(dato)
        bd.desconectar()
        

class ControlListaPag(ControlModeloEntrada):
    '''
    Clase para el control de las Listas Paginadas
    '''

    def cargarDatos(self, listPag, listObj):
        ''' Inserta la lista pasada en listObj en la Lista Paginada listPag. '''
        listPag.setLista(listObj.getLista())


class ControlSeleccionable(ControlModeloEntrada):
    '''
    Clase para el control de Seleccionables
    '''

    def cargarDatos(self, selec, lisObj):
        ''' Inserta la lista pasada en listObj en el Seleccionable selec. '''
        selec.setLista(lisObj.getLista())


class ControlProgreso(object):
    '''
    Clase para el control de Formularios Progreso
    '''

    def accion(self, form, formProg, session, accion):
        
        memprog=MemoriaProgreso(accion+'HAWUVA'+str(session['idProyecto'])+'.txt')
        if memprog.getActual()<memprog.getMaximo():
            por=(memprog.getActual()*100)/memprog.getMaximo()
            formProg.setTituloForm("Realizando acción ... ")
            formProg.setPorcentaje(por)
            resultado=0
        else:
            numDatos=memprog.getTotal()
            if numDatos>=0:
                formProg.setTituloForm("Acción concluida con éxito." )
                formProg.setPorcentaje(100)
                resultado=1
            elif numDatos<0:
                formProg.setTituloForm("La acción ha fracasado.")
                formProg.setPorcentaje(0)
                resultado=1
        del memprog            
        return resultado


class ControlWeb(object):
    '''
    Clase Abstracta para el control de las webs de estadísticas
    '''

    def escribirMenu(self, menu, valor):
        raise NotImplementedError("El método no está implementado en esta clase")

        
class ControlWebIni(ControlWeb):
    '''
    Clase para el control de la web
    '''
    
    def escribirMenu(self, menu, seleccion):
        ''' Controla la acción de mostrar el menú de acciones correspondiente '''
        
        contMenu=ControlMenuFun()
        contMenu.cargarDatos(menu, seleccion)
        del contMenu
    

class ControlWebProyecto(object):
    '''
    Clase para el control de la parte de la web que se dedica a la gestión de proyectos
    '''
    
    def insertar(self, form, formProyect, sesion):
        ''' Controla la acción para recoger los datos e insertar un proyecto si se dan las condiciones '''
        
        idProyecto=0
        resultado=0
        if form.has_key('nombre'):
            formProyect.recogerDatos(form)
            proyect=Proyecto()
            proyect.setDatos(formProyect.getDatos())
            controlAplicacion=ControlAplicacion()
            idProyecto=controlAplicacion.insertar(proyect)
            del controlAplicacion
            del proyect

        if idProyecto>0:
            sesion['idProyecto']=idProyecto
            formProyect.setTituloForm("El siguiente proyecto se ha creado satisfactoriamente. Y está seleccionado.")
            resultado=1
        elif idProyecto==0:
            formProyect.setTituloForm("Inserte los datos del nuevo proyecto:")
            resultado=1
        elif idProyecto==-1:
            formProyect.setTituloForm("El proyecto NO se ha podido crear. Por favor intentelo mas tarde.")
            resultado=0
        else:
            formProyect.setTituloForm("Ya existe un proyecto con el mismo nombre para la misma organización.")
            resultado=0
        return resultado


    def modificar(self, form, formProyect, sesion):
        ''' Controla la acción para recoger los datos y modificar un proyecto si se dan las condiciones '''
         
        codigoTitulo=0
        resultado=0
        idProyecto=sesion['idProyecto']
        proyect=Proyecto()
        controlProyect=ControlProyecto()
        controlProyect.cargarDatos(idProyecto, proyect)
        del controlProyect
        if form.has_key('nombre'):
            formProyect.recogerDatos(form)
            controlAplicacion=ControlAplicacion()
            proyect.setDatos(formProyect.getDatos())
            codigoTitulo=controlAplicacion.modificar(proyect)
            del controlAplicacion
        else:
            formProyect.cogerDatos(proyect.getDatos())
        del proyect
        
        if codigoTitulo>0:
            formProyect.setTituloForm("El proyecto seleccionado se ha modificado satisfactoriamente.")
            resultado=1
        elif codigoTitulo==0:
            formProyect.setTituloForm("Inserte los nuevos datos del proyecto")
            resultado=1
        elif codigoTitulo==-1:
            formProyect.setTituloForm("El proyecto NO se ha podido modificar. Por favor intentelo mas tarde.")
            resultado=0
        else:
            formProyect.setTituloForm("Ya existe otro proyecto con el mismo nombre para la misma organización.")
            resultado=0
        return resultado


    def seleccionar(self, form, formProyect, sesion):
        ''' Controla la acción de seleccionar un proyecto '''
        
        codigoTitulo=0
        idProyecto=0
        
        lisPro=ListaProyectos()
        contLisPro=ControlProyecto()
        contLisPro.listaProyectos(lisPro)
        
        lisPag=ListaPaginada()
        contLisPag=ControlListaPag()
        contLisPag.cargarDatos(lisPag,lisPro)
        
        if form.has_key('seleccionar'):
            formProyect.recogerDatos(form)
            lisPag.setObjetosPagina(formProyect.getObjetosPagina())
            if formProyect.getSelecionado()>0:
                idProyecto=formProyect.getSelecionado()
                sesion['idProyecto']=idProyecto
            else:
                codigoTitulo=1
        elif form.has_key('siguiente'):
            formProyect.recogerDatos(form)
            formProyect.siguientePagina()
        elif form.has_key('anterior'):
            formProyect.recogerDatos(form)
            formProyect.anteriorPagina()
        else:
            lisPag.setObjetosPagina(formProyect.getObjetosPagina())

        pagina=formProyect.getPagina()
        if pagina>=lisPag.numPaginas():
            formProyect.deshabilitarSiguiente()
        else:
            formProyect.habilitarSiguiente()
        if pagina<=1:
            formProyect.deshabilitarAnterior()
        else:
            formProyect.habilitarAnterior()

        lista=lisPag.getListaPagina(pagina)
        formProyect.setLista(lista)
        
        if lisPro.numero()==0:
            formProyect.setTituloForm("No hay ningún proyecto en el sistema.")
        elif codigoTitulo==0:
            if idProyecto>0:
                formProyect.setTituloForm("Ha seleccionado el siguiente proyecto:")
            else: 
                formProyect.setTituloForm("Seleccione un proyecto de la lista:")
        elif codigoTitulo==1:
            formProyect.setTituloForm("NO se ha seleccionado ningún proyecto.")
        
        del lisPro
        del lisPag
        del contLisPro
        del contLisPag


    def eliminar(self, form, formProyect, session):
        ''' Controla la acción para recoger los datos y modificar un proyecto si se dan las condiciones '''
        
        codigoTitulo=0
        borrados=[]
        if form.has_key('eliminar'):
            formProyect.recogerDatos(form)
            controlAplicacion=ControlAplicacion()
            proySel=formProyect.getSeleccionados()
            borrados=controlAplicacion.eliminar(proySel)
            if len(borrados)==len(proySel):
                codigoTitulo=1
            else:
                codigoTitulo=-1
            indiceborra=0
            encontrado=0
            while (indiceborra<len(borrados)) and (encontrado==0):
                if int(borrados[indiceborra])==int(session['idProyecto']):
                    session['idProyecto']=0
                    encontrado=1
                indiceborra=indiceborra+1
            del controlAplicacion
        elif form.has_key('siguiente'):
            formProyect.recogerDatos(form)
            formProyect.siguientePagina()
        elif form.has_key('anterior'):
            formProyect.recogerDatos(form)
            formProyect.anteriorPagina()
       
        lisPro=ListaProyectos()
        contLisPro=ControlProyecto()
        contLisPro.listaProyectos(lisPro)
        
        lisPag=ListaPaginada()
        contLisPag=ControlListaPag()
        contLisPag.cargarDatos(lisPag,lisPro)

        if not form.has_key('siguiente') and not form.has_key('anterior'):
            lisPag.setObjetosPagina(formProyect.getObjetosPagina())

        pagina=formProyect.getPagina()
        while pagina>lisPag.numPaginas() and lisPag.numPaginas()>0:
            pagina=pagina-1
        formProyect.setPagina(pagina)
        if pagina>=lisPag.numPaginas():
            formProyect.deshabilitarSiguiente()
        else:
            formProyect.habilitarSiguiente()
        if pagina<=1:
            formProyect.deshabilitarAnterior()
        else:
            formProyect.habilitarAnterior()

        lista=lisPag.getListaPagina(pagina)
        formProyect.setLista(lista)
         
        if codigoTitulo==0:
            if lisPro.numero()==0:
                formProyect.setTituloForm("No hay ningún proyecto en el sistema.")
            else:
                formProyect.setTituloForm("Seleccione los proyectos que desea eliminar.")
            resultado=1
        elif codigoTitulo==1:
            if lisPro.numero()==0:
                formProyect.setTituloForm("Los proyectos seleccionados se han eliminado satisfactoriamente.<br>No hay ningún proyecto en el sistema.")
            else:
                formProyect.setTituloForm("Los proyectos seleccionados se han eliminado satisfactoriamente.")
            resultado=1
        else:
            formProyect.setTituloForm("Algunos de los proyectos seleccionados NO se han podido eliminar. Por favor intentelo mas tarde.")
            resultado=0
        return resultado
    

class ControlWebLog(object):
    '''
    Clase para el control de la parte de la web que se dedica a la gestión de Logs
    '''

    def insertar(self, form, formDatos, session):
        ''' Controla la acción para recoger los datos del log y guardarlos en un proyecto si se dan las condiciones '''
        
        resultado=0
        if form.has_key('volver'):
            memprog=MemoriaProgreso('DatosHAWUVA'+str(session['idProyecto'])+'.txt')
            if memprog.getActual()<memprog.getMaximo():
                formDatos.setTituloForm("Proceso en curso, recargue la página por favor ... ")
            else:
                numDatos=0 
                numDatos=memprog.getTotal()
                if numDatos>0:
                    formDatos.setTituloForm("Se han introducido "+str(numDatos)+" datos en el proyecto." )
                elif numDatos==0:
                    formDatos.setTituloForm("No se ha introducido ningún dato nuevo.")
                resultado=1
                memprog.eliminar()
            del memprog
        else:
            formDatos.setTituloForm("Introduzca el archivo con los datos y seleccione su tipo.")
        
        lisTipo=ListaTiposBitacora()
        contLisTip=ControlListaTipos()
        contLisTip.cargarDatos(lisTipo)
                
        selec=Seleccionable()
        contSelec=ControlSeleccionable()
        contSelec.cargarDatos(selec,lisTipo)
        formDatos.setTipos(selec)
        return resultado
        

    def guardar(self, form, formDatos, session):
        ''' Controla la acción para guardar los datos en un proyecto si se dan las condiciones '''

        def procesarDatos(archivo, tipo, idProyecto):
            controlAplicacion=ControlAplicacion()
            numDatos = controlAplicacion.insertarDatos(archivo, tipo, idProyecto)
            del controlAplicacion
            
        resultado=0
        if form.has_key('insertar'):
            formDatos.recogerDatos(form)
            tipo=formDatos.getTipo()
            idProyecto=session['idProyecto']
            archivo=formDatos.getArchivo()
            solapar=formDatos.getSolapar()
            controlAplicacion=ControlAplicacion()
            resultado = controlAplicacion.comprobarDatos(archivo, tipo, solapar, idProyecto)
            del controlAplicacion
        if resultado==1:
            p = Process(target=procesarDatos, args=(archivo, tipo, idProyecto, ))
            p.start()
            '''procesarDatos(archivo, tipo, session['idProyecto'])'''
            formDatos.setTituloForm("")
        elif resultado==-1:
            formDatos.setTituloForm("No ha introducido ningún archivo con datos.")
        elif resultado==-2:
            formDatos.setTituloForm("Ya hay un archivo con el mismo nombre en este proyecto.")
        elif resultado==-3:
            formDatos.setTituloForm("El archivo intruducido no contiene datos o no está en el formato indicado.")
        elif resultado==-4:
            formDatos.setTituloForm("El archivo intruducido contiene datos que podrían</br>solaparse con otros datos introduccidos anteriormente.")
        else:
            formDatos.setTituloForm("Se ha producido un error. Pruebe de nuevo.")
        return resultado

    def listar(self, form, formDatos, session):
        ''' Controla la acción para mostrar los logs del proyecto actual '''
        
        resultado=0
        lisLogs=ListaLogs()
        contLisLog=ControlLog()
        contLisLog.listaLogs(lisLogs, session['idProyecto'])
        if lisLogs.numero()>0:
            resultado=1
                    
        lisPag=ListaPaginada()
        contLisPag=ControlListaPag()
        contLisPag.cargarDatos(lisPag,lisLogs)
        
        if form.has_key('siguiente'):
            formDatos.recogerDatos(form)
            formDatos.siguientePagina()
        elif form.has_key('anterior'):
            formDatos.recogerDatos(form)
            formDatos.anteriorPagina()
        
        pagina=formDatos.getPagina()
        if pagina>=lisPag.numPaginas():
            formDatos.deshabilitarSiguiente()
        else:
            formDatos.habilitarSiguiente()
        if pagina<=1:
            formDatos.deshabilitarAnterior()
        else:
            formDatos.habilitarAnterior()

        lista=lisPag.getListaPagina(pagina)
        formDatos.setLista(lista)
        
        if lisLogs.numero()==0:
            formDatos.setTituloForm("Este proyecto no tiene ningún archivo de datos asociado.")
        else:
            formDatos.setTituloForm("Lista de archivos de datos asociados a este proyecto.")
                
        del lisLogs
        del lisPag
        del contLisLog
        del contLisPag
        return resultado
    
    def eliminar(self, form, formLog, session):
        ''' Controla la acción para recoger los datos y modificar un proyecto si se dan las condiciones '''
        
        codigoTitulo=0
        borrados=[]
        if form.has_key('eliminar'):
            formLog.recogerDatos(form)
            controlAplicacion=ControlAplicacion()
            proyLog=formLog.getSeleccionados()
            borrados=controlAplicacion.eliminarDatos(proyLog, session['idProyecto'])
            if len(borrados)==len(proyLog):
                codigoTitulo=1
            else:
                codigoTitulo=-1
            del controlAplicacion
        elif form.has_key('siguiente'):
            formLog.recogerDatos(form)
            formLog.siguientePagina()
        elif form.has_key('anterior'):
            formLog.recogerDatos(form)
            formLog.anteriorPagina()
       
        lisLog=ListaLogs()
        contLisLog=ControlLog()
        contLisLog.listaLogs(lisLog, session['idProyecto'])
        
        lisPag=ListaPaginada()
        contLisPag=ControlListaPag()
        contLisPag.cargarDatos(lisPag,lisLog)

        if not form.has_key('siguiente') and not form.has_key('anterior'):
            lisPag.setObjetosPagina(formLog.getObjetosPagina())

        pagina=formLog.getPagina()
        while pagina>lisPag.numPaginas() and lisPag.numPaginas()>0:
            pagina=pagina-1
        formLog.setPagina(pagina)
        if pagina>=lisPag.numPaginas():
            formLog.deshabilitarSiguiente()
        else:
            formLog.habilitarSiguiente()
        if pagina<=1:
            formLog.deshabilitarAnterior()
        else:
            formLog.habilitarAnterior()

        lista=lisPag.getListaPagina(pagina)
        formLog.setLista(lista)
         
        resultado=1
        if lisLog.numero()==0:
            if codigoTitulo==0:
                formLog.setTituloForm("El proyecto no tiene ningún archivo de datos asociado.")
            else:
                formLog.setTituloForm("Los datos seleccionados se han eliminado satisfactoriamente.<br>No hay ningún archivo asociado a este proyecto.")
        else:
            if codigoTitulo==0:
                formLog.setTituloForm("Seleccione los archivos de datos que desea eliminar.")
            elif codigoTitulo==1:   
                formLog.setTituloForm("Los datos seleccionados se han eliminado satisfactoriamente.")
            else:
                formLog.setTituloForm("Algunos de los archivos de datos seleccionados pueden NO haberse eliminado. Por favor intentelo mas tarde.")
                resultado=0
        return resultado

    
class ControlWebIniEst(ControlWeb):
    '''
    Clase para el control de la web
    '''
    
    def escribirMenu(self, menu, tipo):
        ''' Controla la acción de mostrar el menú de acciones correspondiente '''
        
        menu.borrarAcciones()
        estadistica=Estadistica()
        contEst=ControlEstadistica()
        contEst.cargarDatos(tipo, estadistica)
        menu.setAccionPrincipal(estadistica)
        del contEst
        contMEst=ControlMenuEst()
        contMEst.cargarDatos(menu, tipo)
        del contMEst
        

class ControlWebEst(object):
    '''
    Clase Abstracta para el control de las webs de estadísticas
    '''

    def estadistica(self, form, formTemResumen, session):
        raise NotImplementedError("El método no está implementado en esta clase")
    
        
class ControlWebEstTemResumen(ControlWebEst):
    '''
    Clase para el control de la parte de la web que se dedica al resumen temporal
    '''
    
    def estadistica(self, form, formTemResumen, session):
        
        resultado=0
        idproyecto=session['idProyecto']
        control=ControlLog()
        lista=control.logsFechasLimites(idproyecto)
        if (lista.numero()<>2):
            formTemResumen.setTituloForm('No hay datos para poder realizar los cálculos pertinentes en este proyecto.')
        else:
            logIni=lista.getObjeto(0)
            formTemResumen.setFechaIniLimit(logIni.getInicio())
            logFin=lista.getObjeto(1)
            formTemResumen.setFechaFinLimit(logFin.getFin())
            del logIni
            del logFin
            if form.has_key('mostrar'):
                formTemResumen.recogerDatos(form)
                fechaIni=formTemResumen.getFechaIni()
                fechaFin=formTemResumen.getFechaFin()
                agrupacion=formTemResumen.getAgrupacion()
                del control
                control=ControlEstTemResumen(fechaIni, fechaFin, agrupacion, idproyecto)
                matriz=control.calcularMatriz()
                if len(matriz)==0:
                    formTemResumen.setTituloForm('No hay datos para realizar los calculos entre las fechas proporcionadas.')
                else:
                    formTemResumen.setTituloForm('Si lo desea puede elegir nuevos datos para recalcular:')
                    formTemResumen.setTabla(matriz)
                    resultado=1
            elif form.has_key('generar'):
                formTemResumen.recogerDatos(form)
                fechaIni=formTemResumen.getFechaIni()
                fechaFin=formTemResumen.getFechaFin()
                agrupacion=formTemResumen.getAgrupacion()
                del control
                control=ControlEstTemResumen(fechaIni, fechaFin, agrupacion, idproyecto)
                archivoCSV=control.calcularArchivoCSV()
                if (archivoCSV==""):
                    formTemResumen.setTituloForm('No hay datos para realizar los calculos entre las fechas proporcionadas.')
                else:
                    formTemResumen.setTituloForm('Si lo desea puede elegir nuevos datos para recalcular:') 
                    formTemResumen.setArchivoCSV(archivoCSV)
                    resultado=1
            else:
                formTemResumen.setTituloForm('Seleccione los datos con los que realizar la estadística:')
                resultado=1
        del lista
        del control
        return resultado
    

class ControlWebEstGeoResumen(ControlWebEst):
    '''
    Clase para el control de la parte de la web que se dedica al resumen Geográfico
    '''
    
    def estadistica(self, form, formGeoResumen, session):
        
        resultado=0
        idproyecto=session['idProyecto']
        control=ControlLog()
        lista=control.logsFechasLimites(idproyecto)
        if (lista.numero()<>2):
            formGeoResumen.setTituloForm('No hay datos para poder realizar los cálculos pertinentes en este proyecto.')
        else:
            logIni=lista.getObjeto(0)
            formGeoResumen.setFechaIniLimit(logIni.getInicio())
            logFin=lista.getObjeto(1)
            formGeoResumen.setFechaFinLimit(logFin.getFin())
            del logIni
            del logFin
            if form.has_key('mostrar'):
                formGeoResumen.recogerDatos(form)
                fechaIni=formGeoResumen.getFechaIni()
                fechaFin=formGeoResumen.getFechaFin()
                agrupacion=formGeoResumen.getAgrupacion()
                del control
                control=ControlEstGeoResumen(10, fechaIni, fechaFin, agrupacion, idproyecto)
                matriz=control.calcularMatriz()
                if len(matriz)==0:
                    formGeoResumen.setTituloForm('No hay datos para realizar los calculos entre las fechas proporcionadas.')
                else:
                    formGeoResumen.setTituloForm('Si lo desea puede elegir nuevos datos para recalcular:')
                    formGeoResumen.setTabla(matriz)
                    resultado=1
            elif form.has_key('generar'):
                formGeoResumen.recogerDatos(form)
                fechaIni=formGeoResumen.getFechaIni()
                fechaFin=formGeoResumen.getFechaFin()
                agrupacion=formGeoResumen.getAgrupacion()
                del control
                control=ControlEstGeoResumen(10,fechaIni, fechaFin, agrupacion, idproyecto)
                archivoCSV=control.calcularArchivoCSV()
                if (archivoCSV==""):
                    formGeoResumen.setTituloForm('No hay datos para realizar los calculos entre las fechas proporcionadas.')
                else:
                    formGeoResumen.setTituloForm('Si lo desea puede elegir nuevos datos para recalcular:') 
                    formGeoResumen.setArchivoCSV(archivoCSV)
                    resultado=1
            else:
                formGeoResumen.setTituloForm('Seleccione los datos con los que realizar la estadística:')
                resultado=1
        del lista
        del control
        return resultado
    
    
class ControlWebEstNavInt(ControlWebEst):
    '''
    Clase para el control de la parte de la web que se dedica a las estadisticas de Navegación Interna
    '''
    
    def estadistica(self, form, formPerfil, session):
        
        resultado=0
        idproyecto=session['idProyecto']
        control=ControlLog()
        lista=control.logsFechasLimites(idproyecto)
        if (lista.numero()<>2):
            formPerfil.setTituloForm('No hay datos para poder realizar los cálculos pertinentes en este proyecto.')
        else:
            logIni=lista.getObjeto(0)
            formPerfil.setFechaIniLimit(logIni.getInicio())
            logFin=lista.getObjeto(1)
            formPerfil.setFechaFinLimit(logFin.getFin())
            del logIni
            del logFin
            if form.has_key('mostrar'):
                formPerfil.recogerDatos(form)
                fechaIni=formPerfil.getFechaIni()
                fechaFin=formPerfil.getFechaFin()
                control=ControlEstNavInt(14, fechaIni, fechaFin, idproyecto)
                matriz=control.calcularMatriz()
                if matriz.getNumTransiciones()==0:
                    formPerfil.setTituloForm('No hay datos para realizar los calculos entre las fechas proporcionadas.')
                else:
                    formPerfil.setTituloForm('Si lo desea puede elegir nuevos datos para recalcular:')
                    porcento=control.calcularPorcentaje();
                    formPerfil.setPorcentaje(porcento)
                    mimatriz=TablaCabeceras()
                    mimatriz.setPrimFilCab(matriz.getLlegadas())
                    mimatriz.setPrimColCab(matriz.getSalidas())
                    mimatriz.setTabla(matriz.getTransiciones())
                    formPerfil.setTabla(mimatriz)
                    resultado=1
            elif form.has_key('generar'):
                formPerfil.recogerDatos(form)
                fechaIni=formPerfil.getFechaIni()
                fechaFin=formPerfil.getFechaFin()
                del control
                control=ControlEstNavInt(14, fechaIni, fechaFin, idproyecto)
                archivoCsv=control.calcularArchivoCSV()
                archivoDot=control.calcularArchivoDOT()
                if (archivoCsv=="") or (archivoDot==""):
                    formPerfil.setTituloForm('No hay datos para realizar los calculos entre las fechas proporcionadas.')
                else:
                    formPerfil.setTituloForm('si lo desea puede elegir nuevos datos para recalcular:') 
                    formPerfil.setArchivoCSV(archivoCsv)
                    formPerfil.setArchivoDOT(archivoDot)
                    resultado=1
            else:
                formPerfil.setTituloForm('Seleccione los datos con los que realizar la tabla de transiciones entre páginas:')
                resultado=1
        del lista
        del control
        return resultado

    
class ControlWebEstNavInOut(ControlWebEst):
    '''
    Clase para el control de la parte de la web que se dedica a las estadisticas de Navegación Interna
    '''
    
    def estadistica(self, form, formPerfil, session):
        
        resultado=0
        idproyecto=session['idProyecto']
        control=ControlLog()
        lista=control.logsFechasLimites(idproyecto)
        if (lista.numero()<>2):
            formPerfil.setTituloForm('No hay datos para poder realizar los cálculos pertinentes en este proyecto.')
        else:
            logIni=lista.getObjeto(0)
            formPerfil.setFechaIniLimit(logIni.getInicio())
            logFin=lista.getObjeto(1)
            formPerfil.setFechaFinLimit(logFin.getFin())
            del logIni
            del logFin
            if form.has_key('mostrar'):
                formPerfil.recogerDatos(form)
                fechaIni=formPerfil.getFechaIni()
                fechaFin=formPerfil.getFechaFin()
                del control
                control=ControlEstNavInOut(10, fechaIni, fechaFin, idproyecto)
                matrizIn=control.calcularMatrizEntradas()
                matrizOut=control.calcularMatrizSalidas()
                if len(matrizIn)==0 or len(matrizOut)==0:
                    formPerfil.setTituloForm('No hay datos para realizar los calculos entre las fechas proporcionadas.')
                else:
                    formPerfil.setTituloForm('Si lo desea Puede elegir nuevos datos para recalcular:')
                    formPerfil.setTablaEntradas(matrizIn)
                    formPerfil.setTablaSalidas(matrizOut)
                    resultado=1
            elif form.has_key('generar'):
                formPerfil.recogerDatos(form)
                fechaIni=formPerfil.getFechaIni()
                fechaFin=formPerfil.getFechaFin()
                del control
                control=ControlEstNavInOut(10, fechaIni, fechaFin, idproyecto)
                archivoIn=control.calcularArchivoEntradasCSV()
                archivoOut=control.calcularArchivoSalidasCSV()
                if (archivoIn=="") or (archivoOut==""):
                    formPerfil.setTituloForm('No hay datos para realizar los calculos entre las fechas proporcionadas.')
                else:
                    formPerfil.setTituloForm('Si lo desea puede elegir nuevos datos para recalcular:') 
                    formPerfil.setArchivoEntradas(archivoIn)
                    formPerfil.setArchivoSalidas(archivoOut)
                    resultado=1 
            else:
                formPerfil.setTituloForm('Seleccione los datos con los que realizar la estadística:')
                resultado=1
        del lista
        del control
        return resultado


class ControlWebEstCarDescriptiva(ControlWebEst):
    '''
    Clase para el control de la parte de la web que se dedica a las estadisticas de perfil de navegación
    '''
    
    def estadistica(self, form, formCarga, session):
        
        resultado=0
        idproyecto=session['idProyecto']
        control=ControlLog()
        lista=control.logsFechasLimites(idproyecto)
        if (lista.numero()<>2):
            formCarga.setTituloForm('No hay datos para poder realizar los cálculos pertinentes en este proyecto.')
        else:
            logIni=lista.getObjeto(0)
            formCarga.setFechaIniLimit(logIni.getInicio())
            logFin=lista.getObjeto(1)
            formCarga.setFechaFinLimit(logFin.getFin())
            del logIni
            del logFin
            if form.has_key('mostrar'):
                formCarga.recogerDatos(form)
                fechaIni=formCarga.getFechaIni()
                fechaFin=formCarga.getFechaFin()
                resumen=formCarga.getResumen()
                opcion=formCarga.getOpcion()
                tipo=formCarga.getTipo()
                ceros=formCarga.getCeros()
                control=ControlEstCarDescirptiva(10, fechaIni, fechaFin, tipo, opcion, ceros, idproyecto)
                matriz=control.calcularMatriz()
                if len(matriz)==0:
                    formCarga.setTituloForm('No hay datos para realizar los calculos entre las fechas proporcionadas.')
                else:
                    if resumen==1:
                        resumen=control.calcularResumen()
                        formCarga.setTablaResumen(resumen)
                    porciento=control.calcularPorcentaje()
                    formCarga.setPorcentaje(porciento)
                    formCarga.setTituloForm('Si lo desea puede elegir nuevos datos para recalcular:')
                    formCarga.setTablaGeneral(matriz)
                    resultado=1
            elif form.has_key('generar'):
                formCarga.recogerDatos(form)
                fechaIni=formCarga.getFechaIni()
                fechaFin=formCarga.getFechaFin()
                resumen=formCarga.getResumen()
                opcion=formCarga.getOpcion()
                tipo=formCarga.getTipo()
                ceros=formCarga.getCeros()
                del control
                control=ControlEstCarDescirptiva(10, fechaIni, fechaFin, tipo, opcion, ceros, idproyecto)
                archivoCSV=control.calcularArchivoCSV()
                if (archivoCSV==""):
                    formCarga.setTituloForm('No hay datos para realizar los calculos entre las fechas proporcionadas.')
                else:
                    formCarga.setTituloForm('Si lo desea puede elegir nuevos datos para recalcular:') 
                    formCarga.setArchivoGeneral(archivoCSV)
                    if resumen==1:
                        resumenCSV=control.calcularResumenCSV()
                        formCarga.setArchivoResumen(resumenCSV)
                    resultado=1
            else:
                formCarga.setTituloForm('Seleccione los datos con los que realizar la estadística:')
                resultado=1
        del lista
        del control
        return resultado
    

class ControlWebEstCarModelo(ControlWebEst):
    '''
    Clase para el control de la parte de la web que se dedica a las estadisticas de perfil de navegación
    '''
    
    def estadistica(self, form, formCarga, session):
        
        resultado=0
        idproyecto=session['idProyecto']
        control=ControlLog()
        lista=control.logsFechasLimites(idproyecto)
        if (lista.numero()<>2):
            formCarga.setTituloForm('No hay datos para poder realizar los cálculos pertinentes en este proyecto.')
        else:
            logIni=lista.getObjeto(0)
            formCarga.setFechaIniLimit(logIni.getInicio())
            logFin=lista.getObjeto(1)
            formCarga.setFechaFinLimit(logFin.getFin())
            del logIni
            del logFin
            if form.has_key('mostrar'):
                formCarga.recogerDatos(form)
                fechaIni=formCarga.getFechaIni()
                fechaFin=formCarga.getFechaFin()
                metodo=formCarga.getMetodo()
                metrica=formCarga.getMetrica()
                criterio=formCarga.getCriterio()
                muldistancia=formCarga.getMulDistancia()
                cerosTamano=formCarga.getCerosTamano()
                cerosTiempo=formCarga.getCerosTiempo()
                del control
                control=ControlEstCarModelo(fechaIni, fechaFin, metodo, metrica, criterio, muldistancia, cerosTamano, cerosTiempo, idproyecto)
                matriz=control.calcularMatriz()
                if len(matriz)==0:
                    formCarga.setTituloForm('No hay datos para realizar los calculos entre las fechas proporcionadas.')
                else:
                    formCarga.setTituloForm('Si lo desea puede elegir nuevos datos para recalcular:')
                    formCarga.setTabla(matriz)
                    resultado=1
            elif form.has_key('generar'):
                formCarga.recogerDatos(form)
                fechaIni=formCarga.getFechaIni()
                fechaFin=formCarga.getFechaFin()
                metodo=formCarga.getMetodo()
                metrica=formCarga.getMetrica()
                criterio=formCarga.getCriterio()
                muldistancia=formCarga.getMulDistancia()
                cerosTamano=formCarga.getCerosTamano()
                cerosTiempo=formCarga.getCerosTiempo()
                del control
                control=ControlEstCarModelo(fechaIni, fechaFin, metodo, metrica, criterio, muldistancia, cerosTamano, cerosTiempo, idproyecto)
                archivoCSV=control.calcularArchivoCSV()
                if (archivoCSV==""):
                    formCarga.setTituloForm('No hay datos para realizar los calculos entre las fechas proporcionadas.')
                else:
                    formCarga.setTituloForm('Si lo desea puede elegir nuevos datos para recalcular:') 
                    formCarga.setArchivoCSV(archivoCSV)
                    resultado=1
            else:
                formCarga.setTituloForm('Seleccione los datos con los que realizar la estadística:')
                resultado=1
        del lista
        del control
        return resultado
    

''' ######################################################## '''
''' ################  Paquete modelo  ################ '''
''' ######################################################## '''

import re
from datetime import datetime
    
class Proyecto(object):
    '''
    Clase Proyecto
    '''
    
    def __init__(self):
        ''' Constructor '''
        self.id=0
        self.nombre=""
        self.organizacion=""
        self.descripcion=""

    def getId(self):
        ''' Devuelve el id del proyecto '''
        return self.id

    def getNombre(self):
        ''' Devuelve el nombre del proyecto '''
        return self.nombre

    def getOrganizacion(self):
        ''' Devuelve la organización para la que se realiza el proyecto '''
        return self.organizacion

    def getDescripcion(self):
        ''' Devuelve la descripción del proyecto '''
        return self.descripcion

    def setId(self, id):
        ''' El proyecto toma el id que se pasa como parámetro '''
        self.id=id

    def setNombre(self, nom):
        ''' El proyecto toma el nombre que se pasa como parámetro '''
        self.nombre=nom

    def setOrganizacion(self, org):
        ''' La organización para la que se realiza el proyecto toma el nombre que se pasa como parámetro '''
        self.organizacion=org

    def setDescripcion(self, dcp):
        ''' La descripcion del proyecto toma el valor que se pasa como parámetro '''
        self.descripcion=dcp

    def getDatos(self):
        ''' Devuelve un array con las propiedades del proyecto '''
        return [self.nombre, self.organizacion, self.descripcion]

    def setDatos(self, valores):
        ''' Las propiedades del proyecto toman como valor los datos del array pasado como parámetro '''
        self.nombre=valores[0]
        self.organizacion=valores[1]
        self.descripcion=valores[2]

class Log(object):
    '''
    Clase Log
    '''
    
    def __init__(self):
        ''' Constructor '''
        self.id=0
        self.nombre=""
        self.proyecto=0
        self.fecha=""
        self.inicio=""
        self.fin=""
        self.visitas=0
        self.hits=0
        
    def getId(self):
        ''' Devuelve el id del log'''
        return self.id

    def getNombre(self):
        ''' Devuelve el nombre del Log '''
        return self.nombre

    def getProyecto(self):
        ''' Devuelve el número de proyecto del Log '''
        return self.proyecto

    def getFecha(self):
        ''' Devuelve la fecha de introducción del Log'''
        return self.fecha
    
    def getInicio(self):
        ''' Devuelve la fecha de inicio del Log'''
        return self.inicio
    
    def getFin(self):
        ''' Devuelve la fecha de fin del Log'''
        return self.fin
    
    def getVisitas(self):
        ''' Devuelve el número de visitas del Log'''
        return self.visitas
    
    def getHits(self):
        ''' Devuelve el número de hits del Log'''
        return self.hits
    
    def setId(self, idl):
        ''' El log toma el nombre que se pasa como parámetro '''
        self.id=idl
    
    def setNombre(self, nom):
        ''' El log toma el nombre que se pasa como parámetro '''
        self.nombre=nom

    def setProyecto(self, proy):
        ''' El número de proyecto toma el valor que se pasa como parámetro '''
        self.proyecto=proy

    def setFecha(self, fecha):
        ''' La fecha de introducción del log toma el valor que se pasa como parámetro '''
        self.fecha=fecha
        
    def setInicio(self, fecha):
        ''' La fecha de inicio del log toma el valor que se pasa como parámetro '''
        self.inicio=fecha
        
    def setFin(self, fecha):
        ''' La fecha de fin del log toma el valor que se pasa como parámetro '''
        self.fin=fecha
        
    def setVisitas(self, vist):
        ''' El número de visitas del log toma el valor que se pasa como parámetro '''
        self.visitas=vist
        
    def setHits(self, numhit):
        ''' El número de hits del log toma el valor que se pasa como parámetro '''
        self.hits=numhit
    
    def getDatos(self):
        ''' Devuelve un array con las propiedades del Log '''
        return [self.nombre, self.proyecto, self.fecha, self.inicio, self.fin, self.visitas, self.hits]

    def setDatos(self, valores):
        ''' Las propiedades del Log toman como valor los datos del array pasado como parámetro '''
        self.nombre=valores[0]
        self.proyecto=valores[1]
        self.fecha=valores[2]
        self.inicio=valores[3]
        self.fin=valores[4]
        self.visitas=valores[5]
        self.hits=valores[6]


class Visita(object):
    '''
    Clase Visita
    '''
    
    def __init__(self):
        ''' Constructor '''
        self.proyecto=0
        self.ip=""
        self.inicio=""
        self.fin=""
        self.entrada=""
        self.salida=""

    def getProyecto(self):
        ''' Devuelve el número de proyecto de la visita '''
        return self.proyecto
    
    def getIp(self):
        ''' Devuelve la ip del visitante de la visita '''
        return self.ip

    def getInicio(self):
        ''' Devuelve la fecha de inicio de la visita'''
        return self.inicio
    
    def getFin(self):
        ''' Devuelve la fecha de fin de la visita'''
        return self.fin
    
    def getEntrada(self):
        ''' Devuelve la URI de entrada de la visita'''
        return self.entrada
    
    def getSalida(self):
        ''' Devuelve la URI de salida de la visita'''
        return self.salida
    
    def setProyecto(self, proy):
        ''' El número de proyecto toma el valor que se pasa como parámetro '''
        self.proyecto=proy

    def setIp(self, c_ip):
        ''' La ip del visitanta toma el valor que se pasa como parámetro '''
        self.ip=c_ip
        
    def setInicio(self, fecha):
        ''' La fecha de inicio toma el valor que se pasa como parámetro '''
        self.inicio=fecha
        
    def setFin(self, fecha):
        ''' La fecha de fin toma el valor que se pasa como parámetro '''
        self.fin=fecha
        
    def setEntrada(self, vist_entrada):
        ''' La URI de entrada toma el valor que se pasa como parámetro '''
        self.entrada=vist_entrada
        
    def setSalida(self, visit_salida):
        ''' La URI de salida toma el valor que se pasa como parámetro '''
        self.salida=visit_salida
    
    def getDatos(self):
        ''' Devuelve un array con las propiedades de la visita '''
        return [self.proyecto, self.ip, self.inicio, self.fin, self.entrada, self.salida]

    def setDatos(self, valores):
        ''' Las propiedades de la visita toman como valor los datos del array pasado como parámetro '''
        self.proyecto=valores[0]
        self.ip=valores[1]
        self.inicio=valores[2]
        self.fin=valores[3]
        self.entrada=valores[4]
        self.salida=valores[5]
 
class Hit(object):
    '''
    Clase Hit
    '''
    
    def __init__(self):
        ''' Constructor '''
        self.proyecto=0
        self.s_fecha_hora=""
        self.s_ip=""
        self.cs_metodo=""
        self.cs_uri_sistema_salida=""
        self.cs_uri_sistema_llegada=""
        self.cs_uri_query=""
        self.s_puerto=0
        self.c_ip=""
        self.c_RFC=""
        self.cs_version=""
        self.cs_agente=""
        self.cs_cookie=""
        self.cs_referencia=""
        self.cs_host=""
        self.sc_estado=0
        self.sc_subestado=0
        self.sc_estadoWin32=0
        self.sc_bytes=0
        self.cs_bytes=0
        self.sc_time=0
    
    def getProyecto(self):
        ''' Devuelve el número de proyecto del hit '''
        return self.proyecto
    
    def getFechaHora(self):
        ''' Devuelve la fecha del hit'''
        return self.s_fecha_hora
    
    def getIp(self):
        ''' Devuelve la ip del cliente del hit'''
        return self.c_ip
    
    def getSalida(self):
        ''' Devuelve la URI en la que se produce el hit'''
        return self.cs_uri_sistema_salida
    
    def getLlegada(self):
        ''' Devuelve la URI que solicita el hit'''
        return self.cs_uri_sistema_llegada
    
    def setProyecto(self, proy):
        ''' El número de proyecto toma el valor que se pasa como parámetro '''
        self.proyecto=proy
        
    def setFechaHora(self, fecha):
        ''' La fecha toma el valor que se pasa como parámetro '''
        self.s_fecha_hora=fecha

    def setIp(self, ip):
        ''' La ip del cliente toma el valor que se pasa como parámetro '''
        self.c_ip=ip
        
    def setSalida(self, salida):
        ''' La URI en la que se produce el hit toma el valor que se pasa como parámetro '''
        self.cs_uri_sistema_salida=salida
        
    def setLlegada(self, llegada):
        ''' La URI que solicita el hit toma el valor que se pasa como parámetro '''
        self.cs_uri_sistema_llegada=llegada
    
    def getDatos(self):
        ''' Devuelve un array con las propiedades del hit '''
        vector=[self.proyecto]
        vector.append(self.s_fecha_hora) 
        vector.append(self.s_ip)
        vector.append(self.cs_metodo)
        vector.append(self.cs_uri_sistema_salida)
        vector.append(self.cs_uri_sistema_llegada)
        vector.append(self.cs_uri_query)
        vector.append(self.s_puerto)
        vector.append(self.c_ip)
        vector.append(self.c_RFC)
        vector.append(self.cs_version)
        vector.append(self.cs_agente)
        vector.append(self.cs_cookie)
        vector.append(self.cs_referencia)
        vector.append(self.cs_host)
        vector.append(self.sc_estado)
        vector.append(self.sc_subestado)
        vector.append(self.sc_estadoWin32)
        vector.append(self.sc_bytes)
        vector.append(self.cs_bytes)
        vector.append(self.sc_time)
        return vector

    def setDatos(self, valores):
        ''' Las propiedades del hit tman como valor los datos del array pasado como parámetro '''
        self.proyecto=valores[0]
        self.s_fecha_hora=valores[1] 
        self.s_ip=valores[2]
        self.cs_metodo=valores[3]
        self.cs_uri_sistema_salida=valores[4]
        self.cs_uri_sistema_llegada=valores[5]
        self.cs_uri_query=valores[6]
        self.s_puerto=valores[7]
        self.c_ip=valores[8]
        self.c_RFC=valores[9]
        self.cs_version=valores[10]
        self.cs_agente=valores[11]
        self.cs_cookie=valores[12]
        self.cs_referencia=valores[13]
        self.cs_host=valores[14]
        self.sc_estado=valores[15]
        self.sc_subestado=valores[16]
        self.sc_estadoWin32=valores[17]
        self.sc_bytes=valores[18]
        self.cs_bytes=valores[19]
        self.sc_time=valores[20]


class Visitante(object):
    '''
    Clase Visitante
    '''
    
    def __init__(self):
        ''' Constructor '''
        self.proyecto=0
        self.c_ip=""
        self.c_id_usuario=""
        self.cs_nombre_usuario=""
        self.pais=""
        self.region=""
        self.ciudad=""
        self.latitud=0
        self.longitud=0
    
    def getProyecto(self):
        ''' Devuelve el número de proyecto del visitante '''
        return self.proyecto        
    
    def getIp(self):
        ''' Devuelve la ip del visitante '''
        return self.c_ip
    
    def getIdUsuario(self):
        ''' Devuelve la id del visitante '''
        return self.c_id_usuario
    
    def getNombreUsuario(self):
        ''' Devuelve el nombre del velsitante'''
        return self.cs_nombre_usuario
    
    def getPais(self):
        ''' Devuelve el pais del velsitante'''
        return self.pais
    
    def getRegion(self):
        ''' Devuelve la región del velsitante'''
        return self.region
    
    def getLongitud(self):
        ''' Devuelve la Longitud del lugar donde se encuentra el velsitante'''
        return self.longitud
        
    def getCiudad(self):
        ''' Devuelve la ciudad del velsitante'''
        return self.ciudad
    
    def getLatitud(self):
        ''' Devuelve la Latitud del lugar donde se encuentra el velsitante'''
        return self.latitud

    def setProyecto(self, proy):
        ''' El número de proyecto toma el valor que se pasa como parámetro '''
        self.proyecto=proy
    
    def setIp(self, ip):
        ''' La ip del visitante toma el valor que se pasa como parámetro '''
        self.c_ip=ip
    
    def setIdUsuario(self, idu):
        ''' La id del visitante toma el valor que se pasa como parámetro '''
        self.c_id_usuario=idu
    
    def setNombreUsuario(self, nombusu):
        ''' El nombre del visitante toma el valor que se pasa como parámetro'''
        self.cs_nombre_usuario=nombusu
        
    def setPais(self, pais):
        ''' El pais del visitante toma el valor que se pasa como parámetro'''
        self.pais=pais
    
    def setRegion(self, region):
        ''' La región del visitante toma el valor que se pasa como parámetro'''
        self.region=region
       
    def setCiudad(self, ciudad):
        ''' La ciudad del visitante toma el valor que se pasa como parámetro'''
        self.ciudad=ciudad
        
    def setLatitud(self, lat):
        ''' La Latitud del lugar donde se encuentra el visitante toma el valor que se pasa como parámetro'''
        self.latitud=lat
    
    def setLongitud(self, lon):
        ''' La Longitud del lugar donde se encuentra el visitante toma el valor que se pasa como parámetro'''
        self.longitud=lon
    
    def getDatos(self):
        ''' Devuelve un array con las propiedades del visitante '''
        vector=[self.proyecto]
        vector.append(self.c_ip)
        vector.append(self.c_id_usuario)
        vector.append(self.cs_nombre_usuario)
        vector.append(self.pais)
        vector.append(self.region)
        vector.append(self.ciudad)
        vector.append(self.latitud)
        vector.append(self.longitud)
        return vector

    def setDatos(self, valores):
        ''' Las propiedades del visitante toman como valor los datos del array pasado como parámetro '''
        self.proyecto=valores[0]
        self.c_ip=valores[1]
        self.c_id_usuario=valores[2] 
        self.cs_nombre_usuario=valores[3]
        self.pais=valores[4]
        self.region=valores[5]
        self.ciudad=valores[6]
        self.latitud=valores[7]
        self.longitud=valores[8]
        

class ListaObjetos(object):

    
    def __init__(self):
        ''' Constructor '''
        self.lista=[]
    
    def getObjeto(self, indice):
        return self.lista[indice]

    def getLista(self):
        return self.lista[:]

    def setLista(self, valores):
        self.lista=valores

    def numero(self):
        return len(self.lista)

    def insertar(self, vecObj):
        self.lista.append(vecObj)
    
    def modificar(self, indice, vecObjec):
        if indice < len(self.lista):
            self.lista[indice]=vecObjec
            
    def eliminar(self, indice):
        return self.lista.pop(indice)
    
    def buscar(self, clave):
        ''' Necesita que el objeto tenga getClave() para comparar '''
        numobj=-1
        indiceobj=0
        while indiceobj<len(self.lista) and numobj==-1:
            obj=self.lista[indiceobj]
            if obj.getClave()==clave:
                numvisita=indiceobj
            indiceobj=indiceobj+1
        return numvisita
    
    def escribir(self, archivo):
        ''' Necesita que el objeto tenga getDatos() y el archivo escribir(datos) '''
        numobj=0
        for objeto in self.lista:
            archivo.escribir(objeto.getDatos())
            numobj=numobj+1
        return numobj

    def vaciar(self):
        while 0 < len(self.lista):
            a=self.lista.pop()

    def __del__(self):
        while 0 < len(self.lista):
            a=self.lista.pop()


class ListaProyectos(ListaObjetos):
    '''
    Clase Lista de Proyectos
    '''
    
    def __init__(self):
        ''' Constructor '''
        ListaObjetos.__init__(self)
            
    def getProyectos(self):
        return self.lista[:]
    
    def setProyectos(self, valorPro):
        self.lista=valorPro


class ListaLogs(ListaObjetos):
    '''
    Clase Lista de Logs
    '''
    
    def __init__(self):
        ''' Constructor '''
        ListaObjetos.__init__(self)
        
    def getLogs(self):
        return self.lista[:]
    
    def setLogs(self, valorLog):
        self.lista=valorLog


class ListaVisitas(ListaObjetos):
    '''
    Clase Lista de Visitas
    '''

    def __init__(self):
        ''' Constructor '''
        ListaObjetos.__init__(self)
        
    def getVisita(self, indice):
        return self.lista[indice]

    def getVisitas(self):
        return self.lista[:]

    def setVisitas(self, valores):
        self.lista=valores
        
    def buscar(self, ip):
        numvisita=-1
        indicevisita=0
        while indicevisita<len(self.lista) and numvisita==-1:
            visit=self.lista[indicevisita]
            if visit.getIp()==ip:
                numvisita=indicevisita
            indicevisita=indicevisita+1
        return numvisita
    

class ListaTiposBitacora(ListaObjetos):
    '''
    Clase Lista de Tipos de Bitacoras
    '''
    
    def __init__(self):
        ''' Constructor '''
        ListaObjetos.__init__(self)
    
    def getTipos(self):
        return self.lista[:]
    
    def setTipos(self, valorTip):
        self.lista=valorTip

        
class Bitacora(object):
    '''
    Clase Bitacora general para el siguiente formato generico
    s-date s-time c-ip cs-version cs-method cs-uri-stem sc-status sc-bytes time-taken
    '''
    
    def __init__(self):
        ''' Constructor '''
        self.s_fecha_hora=''
        self.c_ip=''
        self.cs_metodo=''
        self.cs_uri_sistema=''
        self.cs_version=''
        self.sc_estado=0
        self.sc_bytes=0
        self.sc_time=0
    
    def getFechaHora(self):
        return self.s_fecha_hora
    
    def getIp(self):
        return self.c_ip
    
    def getUri(self):
        return self.cs_uri_sistema
        
    def getBitacora(self):
        datos=[]
        datos.append(self.s_fecha_hora)
        datos.append(self.c_ip)
        datos.append(self.cs_metodo)
        datos.append(self.cs_uri_sistema)
        datos.append(self.cs_version)
        datos.append(str(self.estado))
        datos.append(str(self.sc_bytes))
        datos.append(str(self.sc_time))
        return datos

    def setBitacora(self, linea):
        datos=re.match('(\d+-\d+-\d+ \d+:\d+:\d+) ([0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}) (.*) (.*) (HTTP\/.*) (\d+) (\d+) (\d+)', linea)
        self.s_fecha_hora=datos.group(1).strip()
        self.c_ip=datos.group(2).strip()
        self.cs_metodo=datos.group(3).strip()
        self.cs_uri_sistema=datos.group(4).strip()
        self.cs_version=datos.group(5).strip()
        self.sc_estado=int(datos.group(6))
        self.sc_bytes=int(datos.group(7))
        self.sc_time=int(datos.group(8))
    
    def esBitacora(self, linea):
        if re.match('(\d+-\d+-\d+ \d+:\d+:\d+) ([0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}) (.*) (.*) (HTTP\/.*) (\d+) (\d+) (\d+)', linea):
            resultado=True
        else:
            resultado=False
        return resultado
    
    def esPagina(self):
        if re.match('(\S+).(htm|html|php|asp|psp|py)', self.cs_uri_sistema):
            resultado=True
        else:
            resultado=False
        return resultado
    
    def toHit(self, idproyecto):
        hit=Hit()
        vector=[idproyecto]
        vector.append(self.s_fecha_hora.strip()) 
        vector.append("")
        vector.append(self.cs_metodo.strip())
        vector.append("")
        vector.append(self.cs_uri_sistema.strip())
        vector.append("")
        vector.append("")
        vector.append(self.c_ip.strip())
        vector.append("")
        vector.append(self.cs_version.strip())
        vector.append("")
        vector.append("")
        vector.append("")
        vector.append("")
        vector.append(self.sc_estado)
        vector.append("")
        vector.append("")
        vector.append(self.sc_bytes)
        vector.append("")
        vector.append(self.sc_time)
        hit.setDatos(vector)
        return hit


class BitacoraApacheCommon(Bitacora):
    '''
    Clase Bitacora para Apache Common Log Format
    c-ip c-RFC c-user-id [s-date s-time] "cs-method cs-uri-stem cs-version" sc-status sc-bytes time-taken
    '''
    
    def __init__(self):
        ''' Constructor '''
        Bitacora.__init__(self)
        self.c_RFC=''
        self.c_id_usuario=''
    
    def getIdUsuario(self):
        return self.c_id_usuario
    
    def getBitacora(self):
        datos=[]
        datos.append(self.c_ip)
        datos.append(self.c_RFC)
        datos.append(self.c_id_usuario)
        datos.append(self.s_fecha_hora)
        datos.append(self.cs_metodo)
        datos.append(self.cs_uri_sistema)
        datos.append(self.cs_version)
        datos.append(str(self.sc_estado))
        datos.append(str(self.sc_bytes))
        datos.append(str(self.sc_time))
        return datos

    def setBitacora(self, linea):
        datos=re.match('([0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}) (\S+) (\S+) \[([0-9]{1,2}/[a-zA-Z]{3}/[0-9]{4}:[0-9]{1,2}:[0-9]{2}:[0-9]{2}) [+-][0-9]{4}\] "(.*) (.*) (HTTP\/.*)" (\d+) (\d+) (\d+)', linea)
        self.c_ip=datos.group(1).strip()
        self.c_RFC=datos.group(2).strip()
        self.c_id_usuario=datos.group(3).strip()
        self.s_fecha_hora=datetime.strptime(datos.group(4), "%d/%b/%Y:%H:%M:%S").strftime("%Y-%m-%d %H:%M:%S").strip()
        self.cs_metodo=datos.group(5).strip()
        self.cs_uri_sistema=datos.group(6).strip()
        self.cs_version=datos.group(7).strip()
        self.sc_estado=int(datos.group(8))
        self.sc_bytes=int(datos.group(9))
        self.sc_time=int(datos.group(10))
    
    def esBitacora(self, linea):
        if re.match('([0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}) (\S+) (\S+) \[([0-9]{1,2}/[a-zA-Z]{3}/[0-9]{4}:[0-9]{1,2}:[0-9]{2}:[0-9]{2}) [+-][0-9]{4}\] "(.*) (.*) (HTTP\/.*)" (\d+) (\d+) (\d+)', linea):
            resultado=True
        else:
            resultado=False
        return resultado
    
    def toHit(self, idproyecto):
        hit=Hit()
        vector=[idproyecto]
        vector.append(self.s_fecha_hora.strip()) 
        vector.append("")
        vector.append(self.cs_metodo.strip())
        vector.append("")
        vector.append(self.cs_uri_sistema.strip())
        vector.append("")
        vector.append("")
        vector.append(self.c_ip.strip())
        vector.append(self.c_RFC.strip())
        vector.append(self.cs_version.strip())
        vector.append("")
        vector.append("")
        vector.append("")
        vector.append("")
        vector.append(self.sc_estado)
        vector.append("")
        vector.append("")
        vector.append(self.sc_bytes)
        vector.append("")
        vector.append(self.sc_time)
        hit.setDatos(vector)
        return hit


class BitacoraApacheCombined(Bitacora):
    '''
    Clase Bitacora para Apache Combined Log Format:
    c-ip c-RFC c-user-id [s-date s-time] "cs-method cs-uri-stem cs-version" sc-status sc-bytes cs_referer cs_agent time-taken
    '''
    
    def __init__(self):
        ''' Constructor '''
        Bitacora.__init__(self)
        self.c_RFC=''
        self.c_id_usuario=''
        self.cs_referencia=''
        self.cs_agente=''
    
    def getIdUsuario(self):
        return self.c_id_usuario
    
    def getBitacora(self):
        datos=[]
        datos.append(self.c_ip)
        datos.append(self.c_RFC)
        datos.append(self.c_id_usuario)
        datos.append(self.s_fecha_hora)
        datos.append(self.cs_metodo)
        datos.append(self.cs_uri_sistema)
        datos.append(self.cs_version)
        datos.append(str(self.sc_estado))
        datos.append(str(self.sc_bytes))
        datos.append(self.cs_referencia)
        datos.append(self.cs_agente)
        datos.append(str(self.sc_time))
        return datos

    def setBitacora(self, linea):
        datos=re.match('([0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}) (\S+) (\S+) \[([0-9]{1,2}/[a-zA-Z]{3}/[0-9]{4}:[0-9]{1,2}:[0-9]{2}:[0-9]{2}) [+-][0-9]{4}\] "(.*) (.*) (HTTP\/.*)" (\d+) (\d+|-) "(.*)" "(.*)" (\d+)', linea)
        self.c_ip=datos.group(1).strip()
        self.c_RFC=datos.group(2).strip()
        self.c_id_usuario=datos.group(3).strip()
        self.s_fecha_hora=datetime.strptime(datos.group(4), "%d/%b/%Y:%H:%M:%S").strftime("%Y-%m-%d %H:%M:%S").strip()
        self.cs_metodo=datos.group(5).strip()
        self.cs_uri_sistema=datos.group(6).strip()
        self.cs_version=datos.group(7).strip()
        self.sc_estado=int(datos.group(8))
        self.sc_bytes=int(datos.group(9))
        self.cs_referencia=datos.group(10).strip()
        self.cs_agente=datos.group(11).strip()
        self.sc_time=int(datos.group(10))
    
    def esBitacora(self, linea):
        if re.match('([0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}) (\S+) (\S+) \[([0-9]{1,2}/[a-zA-Z]{3}/[0-9]{4}:[0-9]{1,2}:[0-9]{2}:[0-9]{2}) [+-][0-9]{4}\] "(.*) (.*) (HTTP\/.*)" (\d+) (\d+|-) "(.*)" "(.*)" (\d+)', linea):
            resultado=True
        else:
            resultado=False
        return resultado
    
    def toHit(self, idproyecto):
        hit=Hit()
        vector=[idproyecto]
        vector.append(self.s_fecha_hora.strip()) 
        vector.append("")
        vector.append(self.cs_metodo.strip())
        vector.append("")
        vector.append(self.cs_uri_sistema.strip())
        vector.append("")
        vector.append("")
        vector.append(self.c_ip.strip())
        vector.append(self.c_RFC.strip())
        vector.append(self.cs_version.strip())
        vector.append(self.cs_agente.strip())
        vector.append("")
        vector.append(self.cs_referencia.strip())
        vector.append("")
        vector.append(self.sc_estado)
        vector.append("")
        vector.append("")
        vector.append(self.sc_bytes)
        vector.append("")
        vector.append(self.sc_time)
        hit.setDatos(vector)
        return hit


class BitacoraIISMax(Bitacora):
    '''
    Clase Bitacora para W3C Extended Log File de IIS con datos superfluos
    s-date s-time s-sitename s-computername s-ip cs-method cs-uri-stem cs-uri-query s-port cs-username c-ip cs-version cs(User-Agent) cs(Cookie) cs(Referer) cs-host sc-status sc-substatus sc-win32-status sc-bytes cs-bytes time-taken 
    '''
    
    def __init__(self):
        ''' Constructor '''
        Bitacora.__init__(self)
        self.s_nombre_sitio=''
        self.s_nombre_servidor=''
        self.s_ip=''
        self.cs_uri_query=''
        self.s_puerto=80
        self.cs_nombre_usuario=''
        self.cs_agente=''
        self.cs_cookie=''
        self.cs_referencia=''
        self.cs_host=''
        self.sc_subestado=0
        self.sc_estadoWin32=0
        self.cs_bytes=0

    def getNombreUsuario(self):
        return self.cs_nombre_usuario

    def getBitacora(self):
        datos=[]
        datos.append(self.s_fecha_hora)
        datos.append(self.s_nombre_sitio)
        datos.append(self.s_nombre_servidor)
        datos.append(self.s_ip)
        datos.append(self.cs_metodo)
        datos.append(self.cs_uri_sistema)
        datos.append(self.cs_uri_query)
        datos.append(str(self.s_puerto))
        datos.append(self.cs_nombre_usuario)
        datos.append(self.c_ip)
        datos.append(self.cs_version)
        datos.append(self.cs_agente)
        datos.append(self.cs_cookie)
        datos.append(self.cs_referencia)
        datos.append(self.cs_host)
        datos.append(str(self.sc_estado))
        datos.append(str(self.sc_subestado))
        datos.append(str(self.sc_estadoWin32))
        datos.append(str(self.sc_bytes))
        datos.append(str(self.cs_bytes))
        datos.append(str(self.sc_time))
        return datos

    def setBitacora(self, linea):
        datos=re.match('([0-9]{4}-[0-9]{1,2}-[0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2}) (\S+) (\S+) ([0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}) (\S+) (\S+) (\S+) ([0-9]{1,4}) (\S*) ([0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}) (HTTP\/\S+) (\S+) (\S+) (\S+) (\S+) (\d+|-) (\d+|-) (\d+|-) ([0-9]+) (\d+|-) (\d+|-)', linea)
        self.s_fecha_hora=datos.group(1).strip()
        self.s_nombre_sitio=datos.group(2).strip()
        self.s_nombre_servidor=datos.group(3).strip()
        self.s_ip=datos.group(4).strip()
        self.cs_metodo=datos.group(5).strip()
        self.cs_uri_sistema=datos.group(6).strip()
        self.cs_uri_query=datos.group(7).strip()
        self.s_puerto=int(datos.group(8))
        self.cs_nombre_usuario=datos.group(9).strip()
        self.c_ip=datos.group(10).strip()
        self.cs_version=datos.group(11).strip()
        self.cs_agente=datos.group(12).strip()
        self.cs_cookie=datos.group(13).strip()
        self.cs_referencia=datos.group(14).strip()
        self.cs_host=datos.group(15).strip()
        self.sc_estado=int(datos.group(16))
        self.sc_subestado=int(datos.group(17))
        self.sc_estadoWin32=int(datos.group(18))
        self.sc_bytes=int(datos.group(19))
        self.cs_bytes=int(datos.group(20))
        self.sc_time=int(datos.group(21))

    def esBitacora(self, linea):
        if re.match('([0-9]{4}-[0-9]{1,2}-[0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2}) (\S+) (\S+) ([0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}) (\S+) (\S+) (\S+) ([0-9]{1,4}) (\S+) ([0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}) (HTTP\/\S*) (\S+) (\S+) (\S+) (\S+) (\d+|-) (\d+|-) (\d+|-) ([0-9]+) (\d+|-) (\d+|-)', linea):
            resultado=True
        else:
            resultado=False
        return resultado
    
    def toHit(self, idproyecto):
        hit=Hit()
        vector=[idproyecto]
        vector.append(self.s_fecha_hora.strip()) 
        vector.append(self.s_ip.strip())
        vector.append(self.cs_metodo.strip())
        vector.append("")
        vector.append(self.cs_uri_sistema.strip())
        vector.append(self.cs_uri_query.strip())
        vector.append(self.s_puerto)
        vector.append(self.c_ip.strip())
        vector.append("")
        vector.append(self.cs_version.strip())
        vector.append(self.cs_agente.strip())
        vector.append(self.cs_cookie.strip())
        vector.append(self.cs_referencia.strip())
        vector.append(self.cs_host.strip())
        vector.append(self.sc_estado)
        vector.append(self.sc_subestado)
        vector.append(self.sc_estadoWin32)
        vector.append(self.sc_bytes)
        vector.append(self.cs_bytes)
        vector.append(self.sc_time)
        hit.setDatos(vector)
        return hit

class BitacoraIISMin(Bitacora):
    '''
    Clase Bitacora para W3C Extended Log File de IIS sin datos superfluos
    s-date s-time s-ip cs-method cs-uri-stem s-port cs-username c-ip cs-version cs(User-Agent) sc-status sc-substatus sc-bytes time-taken 
    '''
    
    def __init__(self):
        ''' Constructor '''
        Bitacora.__init__(self)
        self.s_ip=''
        self.s_puerto=80
        self.cs_nombre_usuario=''
        self.cs_agente=''
        self.sc_subestado=0
        
    def getNombreUsuario(self):
        return self.cs_nombre_usuario

    def getBitacora(self):
        datos=[]
        datos.append(self.s_fecha_hora)
        datos.append(self.s_ip)
        datos.append(self.cs_metodo)
        datos.append(self.cs_uri_sistema)
        datos.append(str(self.s_puerto))
        datos.append(self.cs_nombre_usuario)
        datos.append(self.c_ip)
        datos.append(self.cs_version)
        datos.append(self.cs_agente)
        datos.append(str(self.sc_estado))
        datos.append(str(self.sc_subestado))
        datos.append(str(self.sc_bytes))
        datos.append(str(self.sc_time))
        return datos

    def setBitacora(self, linea):
        datos=re.match('([0-9]{4}-[0-9]{1,2}-[0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2}) ([0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}) (\S+) (\S+) ([0-9]{1,4}) (\S*) ([0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}) (HTTP\/\S+) (\S+) (\d+|-) (\d+|-) ([0-9]+) (\d+|-)', linea)
        self.s_fecha_hora=datos.group(1).strip()
        self.s_ip=datos.group(2).strip()
        self.cs_metodo=datos.group(3).strip()
        self.cs_uri_sistema=datos.group(4).strip()
        self.s_puerto=int(datos.group(5))
        self.cs_nombre_usuario=datos.group(6).strip()
        self.c_ip=datos.group(7).strip()
        self.cs_version=datos.group(8).strip()
        self.cs_agente=datos.group(9).strip()
        self.sc_estado=int(datos.group(10))
        self.sc_subestado=int(datos.group(11))
        self.sc_bytes=int(datos.group(12))
        self.sc_time=int(datos.group(13))

    def esBitacora(self, linea):
        if re.match('([0-9]{4}-[0-9]{1,2}-[0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2}) ([0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}) (\S+) (\S+) ([0-9]{1,4}) (\S+) ([0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}) (HTTP\/\S*) (\S+) (\d+|-) (\d+|-) ([0-9]+) (\d+|-)', linea):
            resultado=True
        else:
            resultado=False
        return resultado
    
    def toHit(self, idproyecto):
        hit=Hit()
        vector=[idproyecto]
        vector.append(self.s_fecha_hora.strip()) 
        vector.append(self.s_ip.strip())
        vector.append(self.cs_metodo.strip())
        vector.append("")
        vector.append(self.cs_uri_sistema.strip())
        vector.append("")
        vector.append(self.s_puerto)
        vector.append(self.c_ip.strip())
        vector.append("")
        vector.append(self.cs_version.strip())
        vector.append(self.cs_agente.strip())
        vector.append("")
        vector.append("")
        vector.append("")
        vector.append(self.sc_estado)
        vector.append(self.sc_subestado)
        vector.append("")
        vector.append(self.sc_bytes)
        vector.append("")
        vector.append(self.sc_time)
        hit.setDatos(vector)
        return hit
    
''' ######################################################## '''
''' ###################  Paquete control  ################### '''
''' ######################################################## '''

from datetime import datetime, timedelta

class ControlProyecto(object):
    '''
    Clase para el control de los Proyectos
    '''

    def cargarDatos(self, idproyecto, proyecto):
        ''' Carga los datos del proyecto con id idproyecto en el proyecto pasado como parámetro '''
        consulta="select nombre, organizacion, descripcion from proyectos where id="+str(idproyecto)+";"
        bd=BaseDatosWeb()
        respuesta = bd.sentencia(consulta)
        if respuesta.numeroResultados()!=0:
            datos=respuesta.entrada()
            proyecto.setId(idproyecto)
            proyecto.setNombre(datos[0])
            proyecto.setOrganizacion(datos[1])
            proyecto.setDescripcion(datos[2])
        bd.desconectar()

    def getIdProyecto(self, proyecto):
        ''' Obtiene la id del proyecto pasado como parámetro de la base de datos '''
        nombre=proyecto.getNombre()
        organizacion=proyecto.getOrganizacion()
        consulta="select id from proyectos where nombre='"+nombre+"' and organizacion='"+organizacion+"';"
        bd=BaseDatosWeb()
        respuesta = bd.sentencia(consulta)
        if respuesta.numeroResultados()==0:
            resultado=0
        else:
            resultado=respuesta.entrada()[0]
        bd.desconectar()
        return resultado

    def numeroProyectos(self, proyecto):
        ''' Devuelve el número de proyectos iguales al del parámetro en la base de datos '''
        nombre=proyecto.getNombre()
        organizacion=proyecto.getOrganizacion()
        consulta="select id from proyectos where nombre='"+nombre+"' and organizacion='"+organizacion+"';"
        bd=BaseDatosWeb()
        respuesta = bd.sentencia(consulta)
        resultado=respuesta.numeroResultados()
        bd.desconectar()
        return resultado
    
    def numeroProyectosDistintos(self, proyecto):
        ''' Devuelve el número de proyectos iguales al del parámetro proyecto que no coinciden con su id '''
        idproyecto=proyecto.getId()
        nombre=proyecto.getNombre()
        organizacion=proyecto.getOrganizacion()
        consulta="select id from proyectos where nombre='"+nombre+"' and organizacion='"+organizacion+"' and id!="+str(idproyecto)+";"
        bd=BaseDatosWeb()
        respuesta = bd.sentencia(consulta)
        resultado=respuesta.numeroResultados()
        bd.desconectar()
        return resultado
    
    def listaProyectos(self, lisPro):
        ''' Carga la lista de proyectos de la base de datos en el parametro '''
        consulta="select id, nombre, organizacion, descripcion from proyectos where id!=0 order by nombre;"
        bd=BaseDatosWeb()
        respuesta = bd.sentencia(consulta)
        lisPro.vaciar()
        for indiceLisPro in range(respuesta.numeroResultados()):
            dato=respuesta.entrada()
            lisPro.insertar(dato)
        bd.desconectar()

    def insertar(self, proyecto):
        ''' Inserta el proyecto pasado como parámetro en la base de datos '''
        nombre=proyecto.getNombre()
        organizacion=proyecto.getOrganizacion()
        descripcion=proyecto.getDescripcion()
        consulta="insert into proyectos (nombre, organizacion, descripcion) values ('"+nombre+"', '"+organizacion+"','"+descripcion+"');"
        bd=BaseDatosWeb()
        respuesta = bd.sentencia(consulta)
        if respuesta.numeroResultados()==0:
            resultado=0
        else:
            resultado=1
        bd.desconectar()
        return resultado

    def modificar(self, proyecto):
        ''' Modifica el proyecto con los datos del otro parámetro en la base de datos '''
        idproyecto=proyecto.getId()
        nombre=proyecto.getNombre()
        organizacion=proyecto.getOrganizacion()
        descripcion=proyecto.getDescripcion()
        consulta="update proyectos set nombre='"+nombre+"', organizacion='"+organizacion+"', descripcion='"+descripcion+"' where id="+str(idproyecto)
        bd=BaseDatosWeb()
        respuesta = bd.sentencia(consulta)
        if respuesta.numeroResultados()==0:
            resultado=0
        else:
            resultado=1
        bd.desconectar()
        return resultado

    def borrar(self, idproyecto):
        ''' Elimina el proyecto con la id pasada como parámetro '''
        resultado=0
        consulta="delete from proyectos where id="+str(idproyecto)
        bd=BaseDatosWeb()
        respuesta = bd.sentencia(consulta)
        if respuesta.numeroResultados()<>0:
            consulta="delete from logs where proyecto="+str(idproyecto)
            respuesta = bd.sentencia(consulta)
            consulta="delete from visitas where proyecto="+str(idproyecto)
            respuesta = bd.sentencia(consulta)
            consulta="delete from visitantes where proyecto="+str(idproyecto)
            respuesta = bd.sentencia(consulta)
            consulta="delete from hits where proyecto="+str(idproyecto)
            respuesta = bd.sentencia(consulta)
            resultado=1
        bd.desconectar()
        return resultado

class ControlLog(object):
    '''
    Clase para el control de los Logs
    '''

    def cargarDatos(self, nombre, idproyecto, log):
        ''' Carga los datos del log con el nombre pasado por parámetro del proyecto pasado por parametro en el log pasado como parámetro '''
        consulta="select nombre, proyecto, fecha, inicio, fin, visitas, hits from logs where proyecto="+str(idproyecto)+" and nombre='"+nombre+"';"
        bd=BaseDatosWeb()
        respuesta = bd.sentencia(consulta)
        if respuesta.numeroResultados()!=0:
            datos=respuesta.entrada()
            log.setNombre(str(datos[0]))
            log.setProyecto(int(datos[1]))
            log.setFecha(str(datos[2]))
            log.setInicio(str(datos[3]))
            log.setFin(str(datos[4]))
            log.setVisitas(int(datos[5]))
            log.setHits(int(datos[6]))
        bd.desconectar()
    
    def numeroLogs(self, log):
        ''' Devuelve el número de logs con el mismo nombre que hay asociados al mismo proyecto en la base de datos '''
        nombre=log.getNombre()
        idproyecto=log.getProyecto()
        consulta="select proyecto from logs where nombre='"+nombre+"' and proyecto="+str(idproyecto)+";"
        bd=BaseDatosWeb()
        respuesta = bd.sentencia(consulta)
        resultado=respuesta.numeroResultados()
        bd.desconectar()
        return resultado
    
    def numeroLogsFecha(self, log):
        ''' Devuelve el número de logs cuyo intervalo de fechas inicio-fin se solapa con el intervalo de fechas del log introducido '''
        nombre=log.getNombre()
        idproyecto=log.getProyecto()
        consulta="select distinct nombre from logs where proyecto="+str(idproyecto);
        consulta=consulta+" and ((('"+log.getInicio()+"' BETWEEN inicio and fin) or ('"+log.getFin()+"' BETWEEN inicio and fin))"
        consulta=consulta+" or (inicio>'"+log.getInicio()+"' and fin<'"+log.getFin()+"'));"
        bd=BaseDatosWeb()
        respuesta = bd.sentencia(consulta)
        resultado=respuesta.numeroResultados()
        bd.desconectar()
        return resultado
    
    def logsFechasLimites(self, idProyecto):
        ''' Devuelve una lista de 2 logs. En primer lugar el log con datos mas antiguos y      '''
        ''' en segundo lugar el de datos mas recientes del proyecto que se pasa por parametro. '''
        
        lista=ListaLogs()
        bd=BaseDatosWeb()
        consulta="select distinct nombre, proyecto, fecha, inicio, fin, visitas, hits from logs where proyecto="+str(idProyecto)+" and inicio in (select min(inicio) from logs where proyecto="+str(idProyecto)+");"
        respuesta = bd.sentencia(consulta)
        if respuesta.numeroResultados()>0:
            logini=Log()
            dato=respuesta.entrada()
            logini.setDatos(dato)
            lista.insertar(logini)
            consulta="select distinct nombre, proyecto, fecha, inicio, fin, visitas, hits from logs where proyecto="+str(idProyecto)+" and fin in (select max(fin) from logs where proyecto="+str(idProyecto)+");"
            respuesta = bd.sentencia(consulta)
            logfin=Log()
            dato=respuesta.entrada()
            logfin.setDatos(dato)
            lista.insertar(logfin)
        bd.desconectar()
        return lista
    
    def listaLogs(self, lisLog, idProyecto):
        ''' Carga la lista de logs de la base de datos en el parametro '''
        consulta="select nombre, fecha, inicio, fin, visitas, hits from logs where proyecto="+str(idProyecto)+" order by inicio desc;"
        bd=BaseDatosWeb()
        respuesta = bd.sentencia(consulta)
        lisLog.vaciar()
        for indiceLisLog in range(respuesta.numeroResultados()):
            dato=respuesta.entrada()
            lisLog.insertar(dato)
        bd.desconectar()
    
    def insertar(self, log):
        ''' Inserta el log pasado como parámetro en la base de datos '''
        consulta="insert into logs (nombre, proyecto, fecha, inicio, fin, visitas, hits) values "
        consulta=consulta+"('"+log.getNombre()+"', "+str(log.getProyecto())+",'"+log.getFecha()+"','"+log.getInicio()+"','"+log.getFin()+"', "+str(log.getVisitas())+", "+str(log.getHits())+");"
        bd=BaseDatosWeb()
        respuesta = bd.sentencia(consulta)
        if respuesta.numeroResultados()==0:
            resultado=0
        else:
            resultado=1
        bd.desconectar()
        return resultado

    def modificar(self, log):
        ''' Modifica el log con el nombre e idproyecto pasados como parámetro con los nuevos datos en la base de datos'''
        nombre=log.getNombre()
        idproyecto=log.getProyecto()
        consulta="update logs set fecha='"+log.getFecha()+"', inicio='"+log.getInicio()+"', fin='"+log.getFin()+"', visitas="+str(log.getVisitas())+", hits="+str(log.getHits())+" where nombre='"+nombre+"' and proyecto"+str(idproyecto)
        bd=BaseDatosWeb()
        respuesta = bd.sentencia(consulta)
        if respuesta.numeroResultados()==0:
            resultado=0
        else:
            resultado=1
        bd.desconectar()
        return resultado

    def borrar(self, nombre, idproyecto):
        ''' Elimina el log pasado como parámetro junto con todos los datos comprendidos entre su fecha de inicio y fin'''
        log=Log()
        self.cargarDatos(nombre, idproyecto, log)
        consulta="delete from logs where nombre='"+nombre+"' and proyecto="+str(idproyecto)
        bd=BaseDatosWeb()
        respuesta = bd.sentencia(consulta)
        if respuesta.numeroResultados()==0:
            resultado=0
        else:
            resultado=1
            inicio=log.getInicio();
            fin=log.getFin();
            consulta="delete from logs where proyecto="+str(idproyecto)+" and inicio>='"+inicio+"' and fin<='"+fin+"'"
            respuesta = bd.sentencia(consulta)
            consulta="delete from visitas where proyecto="+str(idproyecto)+" and s_fecha_hora_inicio>='"+inicio+"' and s_fecha_hora_fin<='"+fin+"'"
            respuesta = bd.sentencia(consulta)
            consulta="delete from visitantes where proyecto="+str(idproyecto)+" and c_ip not in (select v.c_ip from visitas v where v.proyecto="+str(idproyecto)+")"
            respuesta = bd.sentencia(consulta)
            consulta="delete from hits where proyecto="+str(idproyecto)+" and s_fecha_hora>='"+inicio+"' and s_fecha_hora<='"+fin+"'"
            respuesta = bd.sentencia(consulta)
        bd.desconectar()
        return resultado

    def borrarfechas(self, inicio, fin, idproyecto):
        ''' Elimina todos los logs y datos del proyecto pasado como parámetro entre las fechas pasadas como parámetros de la base de datos'''
        bd=BaseDatosWeb()
        consulta="delete from logs where proyecto="+str(idproyecto)+" and inicio>='"+inicio+"' and fin<='"+fin+"'"
        respuesta = bd.sentencia(consulta)
        consulta="delete from visitantes where proyecto="+str(idproyecto)+" and c_ip not in (select v.c_ip from visitas v where v.proyecto="+str(idproyecto)+")"
        respuesta = bd.sentencia(consulta)
        consulta="delete from visitas where proyecto="+str(idproyecto)+" and s_fecha_hora_inicio>='"+inicio+"' and s_fecha_hora_fin<='"+fin+"'"
        respuesta = bd.sentencia(consulta)
        consulta="delete from hits where proyecto="+str(idproyecto)+" and s_fecha_hora>='"+inicio+"' and s_fecha_hora<='"+fin+"'"
        respuesta = bd.sentencia(consulta)
        bd.desconectar()
        resultado=1
        return resultado

    
class ControlListaTipos(object):
    '''
    Clase para el control de la lista de tipos de bitácoras
    '''

    def cargarDatos(self, lisTipo):
        ''' Carga la lista de tipos de bitacora de la base de datos en el parametro '''
        
        consulta="select id, nombre from tipos_bitacora order by id;"
        bd=BaseDatosWeb()
        respuesta = bd.sentencia(consulta)
        lisTipo.vaciar()
        for indiceTipoPro in range(respuesta.numeroResultados()):
            dato=respuesta.entrada()
            lisTipo.insertar(dato)
        bd.desconectar()


class ControlAplicacion(object):
    '''
    Clase para el Control de la Aplicacion
    '''

    def insertar(self, proyecto):
        ''' Controla la acción de insertar un proyecto '''
        
        idProyecto=0
        controlProyect=ControlProyecto()
        if controlProyect.numeroProyectos(proyecto)==0:
            resultado=controlProyect.insertar(proyecto)
            if resultado==1:
                idProyecto=controlProyect.getIdProyecto(proyecto)
            else:
                idProyecto=-1
        else:
            idProyecto=-2
        del controlProyect
        return idProyecto


    def modificar(self, proyecto):
        ''' Controla la acción para modificar un proyecto si se dan las condiciones '''
         
        resultado=0
        controlProyect=ControlProyecto()
        if controlProyect.numeroProyectosDistintos(proyecto)==0:
            resultado=controlProyect.modificar(proyecto)
            idProyecto=proyecto.getId()
            if resultado==1 and idProyecto!=0:
                resultado=idProyecto
            else:
                resultado=-1
        else:
            resultado=-2
        return resultado


    def eliminar(self, listIdProy):
        ''' Controla la acción para eliminar una lista de proyectos '''
        
        borrados=[]
        controlProyect=ControlProyecto()
        for idp in listIdProy:
            borrado=controlProyect.borrar(idp)
            if borrado==1:
                borrados.append(int(idp))
        del controlProyect
        return borrados

    def comprobarDatos(self, archivo, tipo, solapar, idProyecto):
        ''' Comprueba si se puede realizar la acción para recoger los datos del log y guardarlos en el proyecto '''
        
        resultado=0
        if archivo.file:
            log=Log()
            log.setProyecto(idProyecto)
            nombre=os.path.basename(archivo.filename)
            log.setNombre(nombre)
            controlLog=ControlLog()
            numlogs=controlLog.numeroLogs(log)
            if numlogs==0:
                clector=CreaLector()
                lector=clector.crearlector(tipo)
                fechas=lector.leerFechas(archivo)
                if len(fechas)==2:
                    if solapar==0:
                        log.setInicio(fechas[0])
                        log.setFin(fechas[1])
                        '''fechas=lector.leerFechas(archivo)'''
                        numlogs=controlLog.numeroLogsFecha(log)
                        if numlogs==0:
                            resultado=1
                        else: 
                            resultado=-4
                    else:
                        resultado=1
                else:
                    resultado=-3
                del clector
                del lector
            else:
                resultado=-2
            del controlLog
            del log
        else:
            resultado=-1
        return resultado
    
    def insertarDatos(self, archivo, tipo, idProyecto):
        ''' Controla la acción para recoger los datos del log y guardarlos en un proyecto '''
        
        numlineas=-1
        if archivo.file:
            clector=CreaLector()
            lector=clector.crearlector(tipo)
            numlineas=lector.leer(archivo, idProyecto)        
            del clector
        return numlineas
    
    def eliminarDatos(self, listNomLog, idProyecto):
        ''' Controla la acción para eliminar datos de un proyecto '''
        
        borrados=[]
        controlLog=ControlLog()
        for nombre in listNomLog:
            borrado=controlLog.borrar(nombre, idProyecto)
            if borrado==1:
                borrados.append(nombre)
        del controlLog
        return borrados

    
import hcluster as hclu

        
class ControlEstTemResumen(object):
    '''
    Clase para el Control de la Estadística Resumen Temporal
    '''

    def __init__(self, fechaIni, fechaFin, agrupacion, idproyecto):
        ''' Constructor '''
        ''' agrupacion=0 por hora, agrupacion=1 por día de la semana,'''
        ''' agrupacion=2 por día del mes, agrupacion=3 por mes, agrupacion=4 por año'''

        self.idproyecto=idproyecto
        self.fechaIni=fechaIni
        self.fechaFin=fechaFin
        self.agrupacion=agrupacion
        if agrupacion==0:
            self.funcion="HOUR"
        elif agrupacion==1:
            self.funcion="DAYOFWEEK"
        elif agrupacion==2:
            self.funcion="DAY"
        elif agrupacion==3:
            self.funcion="MONTH"
        else:
            self.funcion="YEAR"
        self.condicionEspecAcc="proyecto="+str(idproyecto)+" and s_fecha_hora>='"+fechaIni+"' and s_fecha_hora<='"+fechaFin+"'"
        self.condicionEspecVis="proyecto="+str(idproyecto)+" and s_fecha_hora_inicio>='"+fechaIni+"' and s_fecha_hora_inicio<='"+fechaFin+"'"
        self.condicionPaginas="((cs_uri_sistema_llegada like '%.html') or (cs_uri_sistema_llegada like '%.htm') or (cs_uri_sistema_llegada like '%.php') or (cs_uri_sistema_llegada like '%.asp') or (cs_uri_sistema_llegada like '%.psp') or (cs_uri_sistema_llegada like '%.py'))"
        self.condicionPagVist="((cs_uri_sistema_entrada like '%.html') or (cs_uri_sistema_entrada like '%.htm') or (cs_uri_sistema_entrada like '%.php') or (cs_uri_sistema_entrada like '%.asp') or (cs_uri_sistema_entrada like '%.psp') or (cs_uri_sistema_entrada like '%.py'))"
        self.condicionNoError="((sc_estado=200) or (sc_estado=206) or (sc_estado=304))"

    def calcularMatriz(self):
        ''' Calcula los matiz con el resumen temporal. '''
                
        matriz=[]
        consuAcc="(select "+self.funcion+"(s_fecha_hora) as agrup, count(cs_uri_sistema_llegada) as accesos from hits "
        consuAcc=consuAcc+"where "+self.condicionEspecAcc+" and "+self.condicionPaginas+" and "+self.condicionNoError+" "
        consuAcc=consuAcc+"group by "+self.funcion+"(s_fecha_hora)) as a"
        consuAcT="(select count(cs_uri_sistema_llegada) as totalacc from hits "
        consuAcT=consuAcT+"where "+self.condicionEspecAcc+" and "+self.condicionPaginas+" and "+self.condicionNoError+" "
        consuAcT=consuAcT+") as b"
        consuVis="(select "+self.funcion+"(s_fecha_hora_inicio) as agrup, count(c_ip) as nvisitas from visitas "
        consuVis=consuVis+"where "+self.condicionEspecVis+" and "+self.condicionPagVist+" "
        consuVis=consuVis+"group by "+self.funcion+"(s_fecha_hora_inicio)) as c"
        consuViT="(select count(c_ip) as totalvis from visitas "
        consuViT=consuViT+"where "+self.condicionEspecVis+" and "+self.condicionPagVist+" "
        consuViT=consuViT+") as d"
        consulta="select a.agrup, accesos, (accesos/totalacc)*100, nvisitas, (nvisitas/totalvis)*100, accesos/nvisitas from "
        consulta=consulta+consuAcc+", "+consuAcT+", "+consuVis+", "+consuViT+" "
        consulta=consulta+"where a.agrup=c.agrup group by a.agrup order by a.agrup;"
        bd=BaseDatosWeb()
        respuesta = bd.sentencia(consulta)
        if respuesta.numeroResultados()!=0:
            for indiceRespuesta in range(respuesta.numeroResultados()):
                dato=respuesta.entrada()
                matriz.append(dato)
        del respuesta 
        bd.desconectar()
        return matriz
    
    
    def calcularArchivoCSV(self):
        ''' Calcula el resumen temporal dejándolo en el archivo cuyo nombre se devuelve '''
                       
        archivoCSV=""
        consuAcc="(select "+self.funcion+"(s_fecha_hora) as agrup, count(cs_uri_sistema_llegada) as accesos from hits "
        consuAcc=consuAcc+"where "+self.condicionEspecAcc+" and "+self.condicionPaginas+" and "+self.condicionNoError+" "
        consuAcc=consuAcc+"group by "+self.funcion+"(s_fecha_hora)) as a"
        consuAcT="(select count(cs_uri_sistema_llegada) as totalacc from hits "
        consuAcT=consuAcT+"where "+self.condicionEspecAcc+" and "+self.condicionPaginas+" and "+self.condicionNoError+" "
        consuAcT=consuAcT+") as b"
        consuVis="(select "+self.funcion+"(s_fecha_hora_inicio) as agrup, count(c_ip) as nvisitas from visitas "
        consuVis=consuVis+"where "+self.condicionEspecVis+" and "+self.condicionPagVist+" "
        consuVis=consuVis+"group by "+self.funcion+"(s_fecha_hora_inicio)) as c"
        consuViT="(select count(c_ip) as totalvis from visitas "
        consuViT=consuViT+"where "+self.condicionEspecVis+" and "+self.condicionPagVist+" "
        consuViT=consuViT+") as d"
        consulta="select a.agrup, accesos, (accesos/totalacc)*100, nvisitas, (nvisitas/totalvis)*100, accesos/nvisitas from "
        consulta=consulta+consuAcc+", "+consuAcT+", "+consuVis+", "+consuViT+" "
        consulta=consulta+"where a.agrup=c.agrup group by a.agrup order by a.agrup;"
        bd=BaseDatosWeb()
        respuesta = bd.sentencia(consulta)
        if respuesta.numeroResultados()!=0:
            archivoCSV="P"+str(self.idproyecto)+"-Res_Temporal-"   
            cabecera=[]
            if self.agrupacion==0:
                cabecera.append("Hora");
                archivoCSV=archivoCSV+"Horas"
            elif self.agrupacion==1:
                cabecera.append("Dia");
                archivoCSV=archivoCSV+"DiaSem"
            elif self.agrupacion==2:
                cabecera.append("Dia");
                archivoCSV=archivoCSV+"DiaMes"
            elif self.agrupacion==3:
                cabecera.append("Mes");
                archivoCSV=archivoCSV+"Meses"
            else:
                cabecera.append("Ano");
                archivoCSV=archivoCSV+"Anos"
            archivoCSV=archivoCSV+"-FecIni"+self.fechaIni+"-FecFin"+self.fechaFin+".csv"
            arch=MemoriaCSV(archivoCSV)
            arch.crear()
            cabecera.append("Accesos");
            cabecera.append("%Accesos");
            cabecera.append("Visitas");
            cabecera.append("%Visitas");
            cabecera.append("Accesos/Visita");
            arch.escribir(cabecera)
            for indiceRespuesta in range(respuesta.numeroResultados()):
                dato=respuesta.entrada()
                arch.escribir(dato)
            del arch
        del respuesta
        bd.desconectar()
        return archivoCSV
    
          
class ControlEstGeoResumen(object):
    '''
    Clase para el Control de la Estadística Resumen Geográfico
    '''

    def __init__(self, numMax, fechaIni, fechaFin, agrupacion, idproyecto):
        ''' Constructor '''
        ''' agrupacion=0 por país, agrupacion=1 por región, agrupacion=2 por ciudad'''

        self.idproyecto=idproyecto
        self.numMaximo=numMax
        self.fechaIni=fechaIni
        self.fechaFin=fechaFin
        self.agrupacion=agrupacion
        if agrupacion==0:
            self.select="apais"
            self.selAgr="pais as apais"
        elif agrupacion==1:
            self.select="apais, aregion"
            self.selAgr="pais as apais, region as aregion"
        else:
            self.select="apais, aregion, aciudad"
            self.selAgr="pais as apais, region as aregion, ciudad as aciudad"
        self.condicionEspecVit="proyecto="+str(idproyecto)
        self.condicionEspecAcc="proyecto="+str(idproyecto)+" and s_fecha_hora>='"+fechaIni+"' and s_fecha_hora<='"+fechaFin+"'"
        self.condicionEspecVis="proyecto="+str(idproyecto)+" and s_fecha_hora_inicio>='"+fechaIni+"' and s_fecha_hora_inicio<='"+fechaFin+"'"
        self.condicionPaginas="((cs_uri_sistema_llegada like '%.html') or (cs_uri_sistema_llegada like '%.htm') or (cs_uri_sistema_llegada like '%.php') or (cs_uri_sistema_llegada like '%.asp') or (cs_uri_sistema_llegada like '%.psp') or (cs_uri_sistema_llegada like '%.py'))"
        self.condicionPagVist="((cs_uri_sistema_entrada like '%.html') or (cs_uri_sistema_entrada like '%.htm') or (cs_uri_sistema_entrada like '%.php') or (cs_uri_sistema_entrada like '%.asp') or (cs_uri_sistema_entrada like '%.psp') or (cs_uri_sistema_entrada like '%.py'))"
        self.condicionNoError="((sc_estado=200) or (sc_estado=206) or (sc_estado=304))"

    def calcularMatriz(self):
        ''' Calcula los matiz con el resumen geográfico. '''
        
        matriz=[]
        consuAcc="(select c_ip as aip, count(c_ip) as naccesos from hits "
        consuAcc=consuAcc+"where "+self.condicionEspecAcc+" and "+self.condicionPaginas+" and "+self.condicionNoError+" group by aip) as z, "
        consuVis="(select c_ip as vip, count(c_ip) as nvisitas from visitas "
        consuVis=consuVis+"where "+self.condicionEspecVis+" and "+self.condicionPagVist+" group by vip) as y, "
        consuAgr="(select c_ip as dip, "+self.selAgr+" from visitantes where "+self.condicionEspecVit+") as x, "
        consuTAc="(select count(c_ip) as tacc from hits "
        consuTAc=consuTAc+"where "+self.condicionEspecAcc+" and "+self.condicionPaginas+" and "+self.condicionNoError+") as w, "
        consuTVi="(select count(c_ip) as tvis FROM visitas "
        consuTVi=consuTVi+"where "+self.condicionEspecVis+" and "+self.condicionPagVist+ ") as v "
        consulta="select "+self.select+", sum(naccesos) as nacc, tacc, sum(nvisitas) as nvis, tvis from "
        consulta=consulta+consuAcc+consuVis+consuAgr+consuTAc+consuTVi
        consulta=consulta+"where aip=vip and dip=vip group by "+self.select+" order by nacc desc;"
        bd=BaseDatosWeb()
        respuesta = bd.sentencia(consulta)
        numrespuesta=respuesta.numeroResultados()
        if numrespuesta!=0:
            indiceRespuesta=0
            if self.numMaximo<numrespuesta:
                sumMaxAcc=0
                sumMaxVis=0
                finmostrar=self.numMaximo-1
            else:
                finmostrar=numrespuesta
            while indiceRespuesta<finmostrar:
                dato=respuesta.entrada()
                fila=[]
                for indag in range(self.agrupacion+1):
                    fila.append(dato[indag])
                fila.append(dato[self.agrupacion+1])
                fila.append(dato[self.agrupacion+1]*100/dato[self.agrupacion+2])
                fila.append(dato[self.agrupacion+3])
                fila.append(dato[self.agrupacion+3]*100/dato[self.agrupacion+4])
                fila.append(dato[self.agrupacion+1]/dato[self.agrupacion+3])
                matriz.append(fila)
                if self.numMaximo<numrespuesta:
                    sumMaxAcc=sumMaxAcc+dato[self.agrupacion+1]
                    sumMaxVis=sumMaxVis+dato[self.agrupacion+3]
                indiceRespuesta=indiceRespuesta+1
            if self.numMaximo<numrespuesta:
                totOtrosAcc=dato[self.agrupacion+2]
                totOtrosVis=dato[self.agrupacion+4]
                sumOtrosAcc=totOtrosAcc-sumMaxAcc
                sumOtrosVis=totOtrosVis-sumMaxVis
                if self.agrupacion==0:
                    otros=["Otros"]
                elif self.agrupacion==1:
                    otros=["Otros","Otros"]
                else:
                    otros=["Otros","Otros","Otros"]
                otros.append(sumOtrosAcc)
                otros.append(sumOtrosAcc*100/totOtrosAcc)
                otros.append(sumOtrosVis)
                otros.append(sumOtrosVis*100/totOtrosVis)
                otros.append(sumOtrosAcc/sumOtrosVis)
                matriz.append(otros)
        del respuesta 
        bd.desconectar()
        return matriz
    
    
    def calcularArchivoCSV(self):
        ''' Calcula el resumen geográfico dejándolo en el archivo cuyo nombre se devuelve '''
                       
        archivoCSV=""
        consuAcc="(select c_ip as aip, count(c_ip) as naccesos from hits "
        consuAcc=consuAcc+"where "+self.condicionEspecAcc+" and "+self.condicionPaginas+" and "+self.condicionNoError+" group by aip) as z, "
        consuVis="(select c_ip as vip, count(c_ip) as nvisitas from visitas "
        consuVis=consuVis+"where "+self.condicionEspecVis+" and "+self.condicionPagVist+" group by vip) as y, "
        consuAgr="(select c_ip as dip, "+self.selAgr+" from visitantes where "+self.condicionEspecVit+") as x, "
        consuTAc="(select count(c_ip) as tacc from hits "
        consuTAc=consuTAc+"where "+self.condicionEspecAcc+" and "+self.condicionPaginas+" and "+self.condicionNoError+") as w, "
        consuTVi="(select count(c_ip) as tvis FROM visitas "
        consuTVi=consuTVi+"where "+self.condicionEspecVis+" and "+self.condicionPagVist+ ") as v "
        consulta="select "+self.select+", sum(naccesos) as nacc, tacc, sum(nvisitas) as nvis, tvis from "
        consulta=consulta+consuAcc+consuVis+consuAgr+consuTAc+consuTVi
        consulta=consulta+"where aip=vip and dip=vip group by "+self.select+" order by nacc desc;"
        bd=BaseDatosWeb()
        respuesta = bd.sentencia(consulta)
        if respuesta.numeroResultados()!=0:
            archivoCSV="P"+str(self.idproyecto)+"-Res_Geografica-"   
            cabecera=[]
            if self.agrupacion==0:
                cabecera.append("Pais");
                archivoCSV=archivoCSV+"Pais"
            elif self.agrupacion==1:
                cabecera.append("Pais");
                cabecera.append("Region");
                archivoCSV=archivoCSV+"Region"
            else:
                cabecera.append("Pais");
                cabecera.append("Region");
                cabecera.append("Ciudad");
                archivoCSV=archivoCSV+"Ciudad"
            archivoCSV=archivoCSV+"-FecIni"+self.fechaIni+"-FecFin"+self.fechaFin+".csv"
            arch=MemoriaCSV(archivoCSV)
            arch.crear()
            cabecera.append("Accesos");
            cabecera.append("%Accesos");
            cabecera.append("Visitas");
            cabecera.append("%Visitas");
            cabecera.append("Accesos/Visita");
            arch.escribir(cabecera)
            for indiceRespuesta in range(respuesta.numeroResultados()):
                dato=respuesta.entrada()
                fila=[]
                for indag in range(self.agrupacion+1):
                    fila.append(dato[indag])
                fila.append(dato[self.agrupacion+1])
                fila.append(dato[self.agrupacion+1]*100/dato[self.agrupacion+2])
                fila.append(dato[self.agrupacion+3])
                fila.append(dato[self.agrupacion+3]*100/dato[self.agrupacion+4])
                fila.append(dato[self.agrupacion+1]/dato[self.agrupacion+3])
                arch.escribir(fila)
            del arch
        del respuesta
        bd.desconectar()
        return archivoCSV
    
                        
class ControlEstNavInt(object):
    '''
    Clase para el Control para las Estadisticas de Navegación Interna
    '''

    def __init__(self, numMax, fechaIni, fechaFin, idproyecto):
        ''' Constructor '''
        self.numAccMaximo=numMax
        self.idproyecto=idproyecto
        self.fechaIni=fechaIni
        self.fechaFin=fechaFin
        self.condicionEspecifica="proyecto="+str(idproyecto)+" and s_fecha_hora>='"+fechaIni+"' and s_fecha_hora<='"+fechaFin+"'"
        self.condicionPaginas="((cs_uri_sistema_llegada like '%.html') or (cs_uri_sistema_llegada like '%.htm') or (cs_uri_sistema_llegada like '%.php') or (cs_uri_sistema_llegada like '%.asp') or (cs_uri_sistema_llegada like '%.psp') or (cs_uri_sistema_llegada like '%.py'))"
        self.condicionNoError="((sc_estado=200) or (sc_estado=206) or (sc_estado=304))"
        self.condicionNumAcces="limit 0 , "+str(numMax-1)
        
    def calcularNumPagTotal(self):
        resultado=0.0
        consulta="select count(cs_uri_sistema_llegada) from hits where "+self.condicionEspecifica+" and "+self.condicionPaginas+" and "+self.condicionNoError+";"
        bd=BaseDatosWeb()
        respuesta = bd.sentencia(consulta)
        dato=respuesta.entrada()
        resultado=dato[0]
        del respuesta
        bd.desconectar()
        return resultado
    
    def calcularPorcentaje(self):
        ''' Calcula el porcentaje de la suma de los accesos a los elementos mas accedidos frente al total de accesos'''
        
        porcento=0.0
        den=self.calcularNumPagTotal()
        if den!=0:
            bd=BaseDatosWeb()
            consulta="select sum(sn_total) from"
            consulta=consulta+" (select cs_uri_sistema_llegada as llegadas_unico, count(cs_uri_sistema_llegada) as sn_total from hits"
            consulta=consulta+" where "+self.condicionEspecifica+" and "+self.condicionPaginas+" and "+self.condicionNoError+" group by llegadas_unico order by sn_total desc "+self.condicionNumAcces+") as A "
            respuesta = bd.sentencia(consulta)
            dato=respuesta.entrada()
            num=dato[0]
            porcento=(float(num)/float(den))*100.0
            del respuesta   
        return porcento

    def calcularMatriz(self):
        ''' Calcula la matriz de transicion entre los elementos mas accedidos '''
        
        matriz=MatrizTransiciones()        
        consulta="select distinct cs_uri_sistema_salida from hits where "+self.condicionEspecifica+" and "+self.condicionPaginas+" and "+self.condicionNoError+";"
        bd=BaseDatosWeb()
        respuesta = bd.sentencia(consulta)
        if respuesta.numeroResultados()!=0:
            consulta=" select distinct H.res_salida, H.res_llegada, H.division, I.ln_total from" 
            consulta=consulta+" (select A.salida_num as res_salida, A.llegada_num as res_llegada, A.numerador/B.divisor as division from" 
            consulta=consulta+" (select cs_uri_sistema_salida as salida_num, cs_uri_sistema_llegada as llegada_num, count(cs_uri_sistema_llegada) as numerador from hits"
            consulta=consulta+" where "+self.condicionEspecifica+" and "+self.condicionPaginas+" and "+self.condicionNoError+" group by llegada_num, salida_num) as A,"
            consulta=consulta+" (select cs_uri_sistema_salida as salida_div, count(cs_uri_sistema_llegada) as divisor from hits"
            consulta=consulta+" where "+self.condicionEspecifica+" and "+self.condicionPaginas+" and "+self.condicionNoError+" group by salida_div) as B "
            consulta=consulta+" where A.salida_num=B.salida_div order by res_salida desc, division desc) as H, "
            consulta=consulta+" (select cs_uri_sistema_llegada as llegada_unico, count(cs_uri_sistema_llegada) as ln_total from hits"
            consulta=consulta+" where "+self.condicionEspecifica+" and "+self.condicionPaginas+" and "+self.condicionNoError+" group by llegada_unico order by ln_total desc "+self.condicionNumAcces+") as I, "
            consulta=consulta+" (select cs_uri_sistema_llegada as salida_unico, count(cs_uri_sistema_llegada) as sn_total from hits"
            consulta=consulta+" where "+self.condicionEspecifica+" and "+self.condicionPaginas+" and "+self.condicionNoError+" group by salida_unico order by sn_total desc "+self.condicionNumAcces+") as J "
            consulta=consulta+" where (H.res_salida=J.salida_unico or H.res_salida='?') and (H.res_llegada=I.llegada_unico) order by I.ln_total desc, H.res_salida"
            respuesta = bd.sentencia(consulta)
            for indiceRespuesta in range(respuesta.numeroResultados()):
                dato=respuesta.entrada()
                matriz.setTransicion(dato[0], dato[1], dato[2])
        del respuesta
        bd.desconectar()
        return matriz
    
    def calcularArchivoDOT(self):
        ''' Calcula la grafo de navegación dejando su representacion en el archivo cuyo nombre se devuelve '''
                       
        archivoDOT=""
        consulta="select distinct cs_uri_sistema_salida from hits where "+self.condicionEspecifica+" and "+self.condicionPaginas+" and "+self.condicionNoError+";"
        bd=BaseDatosWeb()
        respuesta = bd.sentencia(consulta)
        if respuesta.numeroResultados()!=0:
            consulta="select A.salida_num, A.llegada_num, A.numerador/B.divisor as division from" 
            consulta=consulta+" (select cs_uri_sistema_salida as salida_num, cs_uri_sistema_llegada as llegada_num, count(cs_uri_sistema_llegada) as numerador from hits"
            consulta=consulta+" where "+self.condicionEspecifica+" and "+self.condicionPaginas+" and "+self.condicionNoError+" group by llegada_num, salida_num) as A,"
            consulta=consulta+" (select cs_uri_sistema_salida as salida_div, count(cs_uri_sistema_llegada) as divisor from hits"
            consulta=consulta+" where "+self.condicionEspecifica+" and "+self.condicionPaginas+" and "+self.condicionNoError+" group by salida_div) as B" 
            consulta=consulta+" where A.salida_num=B.salida_div order by A.salida_num desc, division desc"
            respuesta = bd.sentencia(consulta)
            archivoDOT="P"+str(self.idproyecto)+"-Nav_Interna-FecIni"+self.fechaIni+"-FecFin"+self.fechaFin+".dot"
            mem=MemoriaDigrafo(archivoDOT)
            mem.crear()
            for indiceRespuesta in range(respuesta.numeroResultados()):
                dato=respuesta.entrada()
                mem.escribirTransicion(dato[0], dato[1], dato[2])
            mem.cerrar()
            del mem
        del respuesta
        bd.desconectar()
        return archivoDOT
    
    def calcularArchivoCSV(self):
        ''' Calcula la matriz de transicion dejándola en el archivo cuyo nombre se devuelve '''
                       
        archivoCSV=""
        consulta="select distinct cs_uri_sistema_salida from hits where "+self.condicionEspecifica+" and "+self.condicionPaginas+" and "+self.condicionNoError+";"
        bd=BaseDatosWeb()
        respuesta = bd.sentencia(consulta)
        if respuesta.numeroResultados()!=0:
            consulta="select A.salida_num, A.llegada_num, A.numerador/B.divisor as division from" 
            consulta=consulta+" (select cs_uri_sistema_salida as salida_num, cs_uri_sistema_llegada as llegada_num, count(cs_uri_sistema_llegada) as numerador from hits"
            consulta=consulta+" where "+self.condicionEspecifica+" and "+self.condicionPaginas+" and "+self.condicionNoError+" group by llegada_num, salida_num) as A,"
            consulta=consulta+" (select cs_uri_sistema_salida as salida_div, count(cs_uri_sistema_llegada) as divisor from hits"
            consulta=consulta+" where "+self.condicionEspecifica+" and "+self.condicionPaginas+" and "+self.condicionNoError+" group by salida_div) as B" 
            consulta=consulta+" where A.salida_num=B.salida_div order by A.salida_num desc, division desc"
            respuesta = bd.sentencia(consulta)
            archivoCSV="P"+str(self.idproyecto)+"-Nav_Interna-FecIni"+self.fechaIni+"-FecFin"+self.fechaFin+".csv"
            matriz= MatrizTransiciones()
            for indiceRespuesta in range(respuesta.numeroResultados()):
                dato=respuesta.entrada()
                matriz.setTransicion(dato[0], dato[1], dato[2])
            mem=MemoriaMatrizTransiciones(archivoCSV)
            mem.escribir(matriz)
            del matriz
            del mem
        del respuesta
        bd.desconectar()
        return archivoCSV
    
    
class ControlEstNavInOut(object):
    '''
    Clase para el Control para las Estadisticas de Navegacion Entrada/Salida
    '''

    def __init__(self, numPagMax, fechaIni, fechaFin, idproyecto):
        ''' Constructor '''
        self.matriz=[]
        self.archivoCSV=""
        self.numMaximo=numPagMax
        self.idproyecto=idproyecto
        self.fechaIni=fechaIni
        self.fechaFin=fechaFin
        self.condicionEspecifica="proyecto="+str(idproyecto)+" and s_fecha_hora_inicio>='"+fechaIni+"' and s_fecha_hora_fin<='"+fechaFin+"'"
        self.condicionPaginas="((cs_uri_sistema_entrada like '%.html') or (cs_uri_sistema_entrada like '%.htm') or (cs_uri_sistema_entrada like '%.php') or (cs_uri_sistema_entrada like '%.asp') or (cs_uri_sistema_entrada like '%.psp') or (cs_uri_sistema_entrada like '%.py'))"

        
    def calcularNumEntTotal(self):
        ''' Calcula el número de veces que se ha entrado en el sitio'''
        
        bd=BaseDatosWeb()
        consulta="select count(cs_uri_sistema_entrada) from visitas"
        consulta=consulta+" where "+self.condicionEspecifica+" and "+self.condicionPaginas+";"
        respuesta = bd.sentencia(consulta)
        dato=respuesta.entrada()
        num=dato[0]
        del respuesta
        bd.desconectar()   
        return num
    
    def calcularNumSalTotal(self):
        ''' Calcula el número de veces que se ha salido del sitio'''
        
        bd=BaseDatosWeb()
        consulta="select count(cs_uri_sistema_salida) from visitas"
        consulta=consulta+" where "+self.condicionEspecifica+" and "+self.condicionPaginas+";"
        respuesta = bd.sentencia(consulta)
        dato=respuesta.entrada()
        num=dato[0]
        del respuesta
        bd.desconectar()   
        return num

    def calcularMatrizEntradas(self):
        ''' Calcula la matriz de entradas mas utilizadas '''
                
        matriz=[]
        den=self.calcularNumEntTotal()
        if den!=0:
            bd=BaseDatosWeb()
            consulta="select distinct cs_uri_sistema_entrada as pagina, count(cs_uri_sistema_entrada) as num_total, count(cs_uri_sistema_entrada)*100/"+str(den)+" as porcentaje from visitas"
            consulta=consulta+" where "+self.condicionEspecifica+" and "+self.condicionPaginas+" group by pagina order by num_total desc;"
        respuesta = bd.sentencia(consulta)
        numrespuesta=respuesta.numeroResultados()
        if numrespuesta!=0:
            indiceRespuesta=0
            if self.numMaximo<numrespuesta:
                sumMaxAcc=0
                finmostrar=self.numMaximo-1
            else:
                finmostrar=numrespuesta
            while indiceRespuesta<finmostrar:
                dato=respuesta.entrada()
                matriz.append(dato)
                if self.numMaximo<numrespuesta:
                    sumMaxAcc=sumMaxAcc+dato[1]
                indiceRespuesta=indiceRespuesta+1
            if self.numMaximo<numrespuesta:
                totOtrosAcc=float(self.calcularNumEntTotal())
                sumOtrosAcc=float(totOtrosAcc-sumMaxAcc)
                otros=["Otras"]
                otros.append(sumOtrosAcc)
                otros.append(sumOtrosAcc*100/totOtrosAcc)
                matriz.append(otros)        
            del respuesta
            bd.desconectar()
        return matriz [:]
    
    def calcularMatrizSalidas(self):
        ''' Calcula la matriz de salidas mas utilizadas '''
                
        matriz=[]
        den=self.calcularNumSalTotal()
        if den!=0:
            bd=BaseDatosWeb()
            consulta="select distinct cs_uri_sistema_salida as pagina, count(cs_uri_sistema_salida) as num_total, count(cs_uri_sistema_salida)*100/"+str(den)+" as porcentaje from visitas"
            consulta=consulta+" where "+self.condicionEspecifica+" and "+self.condicionPaginas+" group by pagina order by num_total desc;"
        respuesta = bd.sentencia(consulta)
        numrespuesta=respuesta.numeroResultados()
        if numrespuesta!=0:
            indiceRespuesta=0
            if self.numMaximo<numrespuesta:
                sumMaxAcc=0
                finmostrar=self.numMaximo-1
            else:
                finmostrar=numrespuesta
            while indiceRespuesta<finmostrar:
                dato=respuesta.entrada()
                matriz.append(dato)
                if self.numMaximo<numrespuesta:
                    sumMaxAcc=sumMaxAcc+dato[1]
                indiceRespuesta=indiceRespuesta+1
            if self.numMaximo<numrespuesta:
                totOtrosAcc=float(self.calcularNumSalTotal())
                sumOtrosAcc=float(totOtrosAcc-sumMaxAcc)
                otros=["Otras"]
                otros.append(sumOtrosAcc)
                otros.append(sumOtrosAcc*100/totOtrosAcc)
                matriz.append(otros)        
            del respuesta
            bd.desconectar()
        return matriz [:]
    
    
    def calcularArchivoEntradasCSV(self):
        ''' Calcula la matriz de entradas con el número total y los porcentajes dejandola en el archivo cuyo nombre se devuelve '''
                
        archivoCSV=""
        den=self.calcularNumEntTotal()
        if den!=0:
            bd=BaseDatosWeb()
            consulta="select distinct cs_uri_sistema_entrada as pagina, count(cs_uri_sistema_entrada) as num_total, count(cs_uri_sistema_entrada)*100/"+str(den)+" as porcentaje from visitas"
            consulta=consulta+" where "+self.condicionEspecifica+" and "+self.condicionPaginas+" group by pagina order by num_total desc;"
            respuesta = bd.sentencia(consulta)
            archivoCSV="P"+str(self.idproyecto)+"-Nav_Entradas-FecIni"+self.fechaIni+"-FecFin"+self.fechaFin+".csv"
            arch=MemoriaCSV(archivoCSV)
            arch.crear()
            arch.escribir(["Entrada","Accesos","Porcentaje"])
            for indiceRespuesta in range(respuesta.numeroResultados()):
                dato=respuesta.entrada()
                arch.escribir(dato)
            del respuesta
            del arch
            bd.desconectar()
        return archivoCSV
    
    
    def calcularArchivoSalidasCSV(self):
        ''' Calcula la matriz de salidas con el número total y los porcentajes dejandola en el archivo cuyo nombre se devuelve '''
                
        self.archivoCSV=""
        den=self.calcularNumSalTotal()
        if den!=0:
            bd=BaseDatosWeb()
            consulta="select distinct cs_uri_sistema_salida as pagina, count(cs_uri_sistema_salida) as num_total, count(cs_uri_sistema_salida)*100/"+str(den)+" as porcentaje from visitas"
            consulta=consulta+" where "+self.condicionEspecifica+" and "+self.condicionPaginas+" group by pagina order by num_total desc;"
            respuesta = bd.sentencia(consulta)
            archivoCSV="P"+str(self.idproyecto)+"-Nav_Salidas-FecIni"+self.fechaIni+"-FecFin"+self.fechaFin+".csv"
            arch=MemoriaCSV(archivoCSV)
            arch.crear()
            arch.escribir(["Salida","Accesos","Porcentaje"])
            for indiceRespuesta in range(respuesta.numeroResultados()):
                dato=respuesta.entrada()
                arch.escribir(dato)
            del respuesta
            del arch
            bd.desconectar()
        return archivoCSV


class ControlEstCarModelo(object):
    '''
    Clase para el Control para las Estadísticas de Caracterización de la Carga
    '''
    def __init__(self, fechaIni, fechaFin, metodo, metrica, criterio, muldistancia, cerosTamano, cerosTiempo, idproyecto):
        ''' Constructor '''
        self.idproyecto=idproyecto
        self.fechaIni=fechaIni
        self.fechaFin=fechaFin
        misMetodos=["single","complete","average","weighted"]
        if metodo>=len(misMetodos) or metodo<0:
            self.metodo="centroid"
        else:
            self.metodo=misMetodos[metodo]
        misMetricas=["euclidean","minkowski","cityblock"]
        if metrica>=len(misMetricas) or metrica<0:
            self.metrica="euclidean"
        else:
            self.metrica=misMetricas[metrica]
        misCriterios=["distance","maxclust","inconsistent"]
        if criterio>=len(misCriterios) or criterio<0:
            self.criterio="distance"
        else:
            self.criterio=misCriterios[criterio]
        if muldistancia<=0 or muldistancia>1:
            self.muldistancia=0.5
        else:
            self.muldistancia=muldistancia
        self.condicionCeros=""
        if cerosTamano<>0 or cerosTiempo<>0:
            self.condicionCeros="("
            if cerosTamano<>0:
                self.condicionCeros=self.condicionCeros+"sc_bytes<>0"
                self.ceros=1
            if cerosTiempo<>0:
                self.ceros=2
                if cerosTamano<>0:
                    self.condicionCeros=self.condicionCeros+" and "
                    self.ceros=3                    
                self.condicionCeros=self.condicionCeros+"sc_time<>0"
            self.condicionCeros=self.condicionCeros+")"
        else:
            self.ceros=0
        self.condicionEspecifica="proyecto="+str(idproyecto)+" and s_fecha_hora>='"+fechaIni+"' and s_fecha_hora<='"+fechaFin+"'"
        self.condicionPaginas="((cs_uri_sistema_llegada like '%.html') or (cs_uri_sistema_llegada like '%.htm') or (cs_uri_sistema_llegada like '%.php') or (cs_uri_sistema_llegada like '%.asp') or (cs_uri_sistema_llegada like '%.psp') or (cs_uri_sistema_llegada like '%.py'))"
        self.condicionNoError="((sc_estado=200) or (sc_estado=206) or (sc_estado=304))"

    def calcularMatriz(self):
        ''' Calcula la matiz con los representantes de la agrupación de carga.''' 
                
        matriz=[]
        consulta="select cs_uri_sistema_llegada as archivo, count(cs_uri_sistema_llegada) as accesos,"
        consulta=consulta+" avg(sc_bytes) as tamano,"
        consulta=consulta+" avg(sc_time) as tiempo from hits" 
        consulta=consulta+" where "+self.condicionEspecifica+" and "+self.condicionPaginas+" and "+self.condicionNoError
        if self.ceros>0:
            consulta=consulta+" and "+self.condicionCeros
        consulta=consulta+" group by cs_uri_sistema_llegada;"
        bd=BaseDatosWeb()
        respuesta = bd.sentencia(consulta)
        if respuesta.numeroResultados()!=0:
            uris=[]
            accesos=[]
            tamano=[]
            tiempo=[]
            valores=[]
            for indiceRespuesta in range(respuesta.numeroResultados()):
                dato=respuesta.entrada()
                uris.append(dato[0])
                accesos.append(dato[1])
                tamano.append(dato[2])
                tiempo.append(dato[3])
                valores.append([dato[1],dato[2],dato[3]])
            y=hclu.pdist(valores, metric=self.metrica)
            Z = hclu.linkage(y, method=self.metodo, metric=self.metrica)
            agrupacion = hclu.fcluster(Z,  self.muldistancia*y.max(), criterion=self.criterio)
            numgrup={}
            sumacc={}
            sumtam={}
            sumtie={}
            for indiceUris in range(len(uris)):
                if numgrup.has_key(str(agrupacion[indiceUris])):
                    numgrup[str(agrupacion[indiceUris])]=numgrup[str(agrupacion[indiceUris])]+1
                    sumacc[str(agrupacion[indiceUris])]=sumacc[str(agrupacion[indiceUris])]+float(accesos[indiceUris])
                    sumtam[str(agrupacion[indiceUris])]=sumtam[str(agrupacion[indiceUris])]+float(tamano[indiceUris])
                    sumtie[str(agrupacion[indiceUris])]=sumtie[str(agrupacion[indiceUris])]+float(tiempo[indiceUris])
                else:
                    numgrup[str(agrupacion[indiceUris])]=1
                    sumacc[str(agrupacion[indiceUris])]=float(accesos[indiceUris])
                    sumtam[str(agrupacion[indiceUris])]=float(tamano[indiceUris])
                    sumtie[str(agrupacion[indiceUris])]=float(tiempo[indiceUris])
            red=hclu.leaders(Z, agrupacion)
            repres=red[0]
            idgrup=red[1]
            representantes=[]
            for indiceGrupos in range(len(idgrup)):
                idrep=repres[indiceGrupos]
                if idrep>=len(uris):
                    minumgrup=numgrup[str(idgrup[indiceGrupos])]
                    mediaacc=sumacc[str(idgrup[indiceGrupos])]/(minumgrup*1.0)
                    mediatam=sumtam[str(idgrup[indiceGrupos])]/(minumgrup*1.0)
                    mediatie=sumtie[str(idgrup[indiceGrupos])]/(minumgrup*1.0)
                    representantes.append([idgrup[indiceGrupos], minumgrup, "-", '{:.2f}'.format(mediaacc), '{:.2f}'.format(mediatam), '{:.2f}'.format(mediatie)])
                else:
                    representantes.append([idgrup[indiceGrupos], numgrup[str(idgrup[indiceGrupos])], uris[idrep], '{:.0f}'.format(accesos[idrep]), '{:.2f}'.format(tamano[idrep]), '{:.2f}'.format(tiempo[idrep])])
        del respuesta
        bd.desconectar()
        return representantes
    
    def calcularArchivoCSV(self):
        ''' Calcula con la agrupacion de ficheros '''
                       
        archivoCSV=""
        consulta="select cs_uri_sistema_llegada as archivo, count(cs_uri_sistema_llegada) as accesos,"
        consulta=consulta+" avg(sc_bytes) as tamano,"
        consulta=consulta+" avg(sc_time) as tiempo from hits" 
        consulta=consulta+" where "+self.condicionEspecifica+" and "+self.condicionPaginas+" and "+self.condicionNoError
        if self.ceros>0:
            consulta=consulta+" and "+self.condicionCeros
        consulta=consulta+" group by cs_uri_sistema_llegada;"
        bd=BaseDatosWeb()
        respuesta = bd.sentencia(consulta)
        if respuesta.numeroResultados()!=0:
            uris=[]
            accesos=[]
            tamano=[]
            tiempo=[]
            valores=[]
            for indiceRespuesta in range(respuesta.numeroResultados()):
                dato=respuesta.entrada()
                uris.append(dato[0])
                accesos.append(dato[1])
                tamano.append(dato[2])
                tiempo.append(dato[3])
                valores.append([dato[1],dato[2],dato[3]])
            y=hclu.pdist(valores, metric=self.metrica)
            Z = hclu.linkage(y, method=self.metodo, metric=self.metrica)
            T = hclu.fcluster(Z,  self.muldistancia*y.max(), criterion=self.criterio)
            archivoCSV=self.nombreArchivo()
            arch=MemoriaCSV(archivoCSV)
            arch.crear()
            arch.escribir(["Grupo", "Archivo", "Accesos", "Tamano", "Tiempo"])
            for indiceUris in range(len(uris)):
                arch.escribir([T[indiceUris], uris[indiceUris], accesos[indiceUris], tamano[indiceUris], tiempo[indiceUris]])
            del arch
        del respuesta
        bd.desconectar()
        return archivoCSV
    
    def nombreArchivo(self):
        nombre="P"+str(self.idproyecto)+"-Agrupacion_Carga_"+self.metodo+"_"+self.metrica+"_"+self.criterio
        nombre=nombre+"_"+'{:.4f}'.format(self.muldistancia)+"_"+str(self.ceros)+"-FecIni"+self.fechaIni+"-FecFin"+self.fechaFin+".csv"
        return nombre


class ControlEstCarDescirptiva(object):
    '''
    Clase para el Control de la Estadística de Carga Descriptiva
    '''

    def __init__(self, numMax, fechaIni, fechaFin, tipo, opcion, ceros, idproyecto):
        ''' Constructor '''
        ''' opcion=0 caracteristica Accesos, opcion=1 caracteristica Tamaños, opcion=2 caracteristica Tiempo'''
        ''' tipo=0 datos menores, tipo=1 datos mayores '''
        ''' ceros=0 solo de valores ceros, ceros=1 con ceros, ceros=2 sin ceros '''
        
        self.numPagMaximo=numMax
        self.idproyecto=idproyecto
        self.fechaIni=fechaIni
        self.fechaFin=fechaFin
        self.tipo=tipo
        self.opcion=opcion
        self.ceros=ceros
        if tipo==0:
            condicionOrdenPre="DESC"
        else:
            condicionOrdenPre="ASC"
        if opcion==0:
            self.condicionOrden="order by accesos "+condicionOrdenPre
            condicionCerosPre=""
        elif opcion==1:
            if ceros==0:
                self.condicionOrden="order by accesos DESC"                
            elif ceros>0:
                self.condicionOrden="order by tamano "+condicionOrdenPre
            condicionCerosPre="sc_bytes"
        else:
            if ceros==0:
                self.condicionOrden="order by accesos DESC"               
            elif ceros>0:
                self.condicionOrden="order by tiempo "+condicionOrdenPre
            condicionCerosPre="sc_time"
        if opcion==0:
            self.condicionCeros=""
        else:
            if ceros==0:
                self.condicionCeros=condicionCerosPre+"=0"                
            elif ceros==1:
                self.condicionCeros="("+condicionCerosPre+"=0 or "+condicionCerosPre+"<>0)" 
            else:
                self.condicionCeros=condicionCerosPre+"<>0"
        self.condicionEspecifica="proyecto="+str(idproyecto)+" and s_fecha_hora>='"+fechaIni+"' and s_fecha_hora<='"+fechaFin+"'"
        self.condicionPaginas="((cs_uri_sistema_llegada like '%.html') or (cs_uri_sistema_llegada like '%.htm') or (cs_uri_sistema_llegada like '%.php') or (cs_uri_sistema_llegada like '%.asp') or (cs_uri_sistema_llegada like '%.psp') or (cs_uri_sistema_llegada like '%.py'))"
        self.condicionNoError="((sc_estado=200) or (sc_estado=206) or (sc_estado=304))"
        self.condicionNumPag="limit 0 , "+str(numMax)
        
    def calcularNumPagTotal(self):
        resultado=0.0
        consulta="select sum(accesos) from "
        consulta=consulta+"(select cs_uri_sistema_llegada as archivo, count(cs_uri_sistema_llegada) as accesos, avg(sc_bytes) as tamano, avg(sc_time) as tiempo from hits" 
        consulta=consulta+" where "+self.condicionEspecifica+" and "+self.condicionPaginas+" and "+self.condicionNoError
        if self.opcion<>0:
                consulta=consulta+" and "+self.condicionCeros
        consulta=consulta+" group by cs_uri_sistema_llegada) as A;"
        bd=BaseDatosWeb()
        respuesta = bd.sentencia(consulta)
        dato=respuesta.entrada()
        resultado=dato[0]
        del respuesta
        bd.desconectar()
        return resultado
    
    def calcularPorcentaje(self):
        ''' Calcula el porcentaje de accesos a numPagMax páginas. '''
        
        porcento=0.0
        den=self.calcularNumPagTotal()
        if den!=0:
            consulta="select sum(accesos) from "
            consulta=consulta+"(select cs_uri_sistema_llegada as archivo, count(cs_uri_sistema_llegada) as accesos, avg(sc_bytes) as tamano, avg(sc_time) as tiempo from hits" 
            consulta=consulta+" where "+self.condicionEspecifica+" and "+self.condicionPaginas+" and "+self.condicionNoError
            if self.opcion<>0:
                consulta=consulta+" and "+self.condicionCeros
            consulta=consulta+" group by cs_uri_sistema_llegada "+self.condicionOrden+" "+self.condicionNumPag+") as A;"
            bd=BaseDatosWeb()
            respuesta = bd.sentencia(consulta)
            dato=respuesta.entrada()
            num=dato[0]
            porcento=(float(num)/float(den))*100
            del respuesta
            bd.desconectar()   
        return porcento
    
    def calcularMatriz(self):
        ''' Calcula los matiz de carga descriptiva por página con los datos mas relevantes. '''
                
        matriz=[]
        consulta="select cs_uri_sistema_llegada as archivo, count(cs_uri_sistema_llegada) as accesos,"
        if (self.ceros<>0) or (self.opcion<>1):
            consulta=consulta+" min(sc_bytes), max(sc_bytes),"
        consulta=consulta+" avg(sc_bytes) as tamano,"
        if (self.ceros<>0) or (self.opcion<>2):
            consulta=consulta+" min(sc_time), max(sc_time),"
        consulta=consulta+" avg(sc_time) as tiempo from hits" 
        consulta=consulta+" where "+self.condicionEspecifica+" and "+self.condicionPaginas+" and "+self.condicionNoError
        if self.opcion<>0:
                consulta=consulta+" and "+self.condicionCeros
        consulta=consulta+" group by cs_uri_sistema_llegada "+self.condicionOrden+" "+self.condicionNumPag+";"
        bd=BaseDatosWeb()
        respuesta = bd.sentencia(consulta)
        if respuesta.numeroResultados()!=0:
            for indiceRespuesta in range(respuesta.numeroResultados()):
                dato=respuesta.entrada()
                matriz.append(dato)
        del respuesta 
        bd.desconectar()
        return matriz
    
    
    def calcularArchivoCSV(self):
        ''' Calcula la matriz de carga descriptiva por página dejándola en el archivo cuyo nombre se devuelve '''
                       
        archivoCSV=""
        consulta="select cs_uri_sistema_llegada as archivo, count(cs_uri_sistema_llegada) as accesos,"
        cabecera=["Página","Accesos"]
        if (self.ceros<>0) or (self.opcion<>1):
            consulta=consulta+" min(sc_bytes), max(sc_bytes),"
            cabecera.append("Min(Tamaño)")
            cabecera.append("Max(Tamaño)")
        consulta=consulta+" avg(sc_bytes) as tamano,"
        cabecera.append("Tamaño")
        if (self.ceros<>0) or (self.opcion<>2):
            consulta=consulta+" min(sc_time), max(sc_time),"
            cabecera.append("Min(Tiempo)")
            cabecera.append("Max(Tiempo)")
        consulta=consulta+" avg(sc_time) as tiempo from hits"
        cabecera.append("Tiempo") 
        consulta=consulta+" where "+self.condicionEspecifica+" and "+self.condicionPaginas+" and "+self.condicionNoError
        if self.opcion<>0:
                consulta=consulta+" and "+self.condicionCeros
        consulta=consulta+" group by cs_uri_sistema_llegada "+self.condicionOrden+";"
        bd=BaseDatosWeb()
        respuesta = bd.sentencia(consulta)
        if respuesta.numeroResultados()!=0:
            archivoCSV=self.nombreArchivo()
            arch=MemoriaCSV(archivoCSV)
            arch.crear()
            arch.escribir(cabecera)
            for indiceRespuesta in range(respuesta.numeroResultados()):
                dato=respuesta.entrada()
                arch.escribir(dato)
            del arch
        del respuesta
        bd.desconectar()
        return archivoCSV
    
    def calcularResumen(self):
        ''' Calcula los matiz resumen de carga descriptiva con los datos mas relevantes.''' 
                
        matriz=[]
        consulta="select cs_uri_sistema_llegada as archivo, count(cs_uri_sistema_llegada) as accesos,"
        consulta=consulta+" avg(sc_bytes) as tamano,"
        consulta=consulta+" avg(sc_time) as tiempo from hits" 
        consulta=consulta+" where "+self.condicionEspecifica+" and "+self.condicionPaginas+" and "+self.condicionNoError
        if self.opcion<>0:
                consulta=consulta+" and "+self.condicionCeros
        consulta=consulta+" group by cs_uri_sistema_llegada;"
        bd=BaseDatosWeb()
        respuesta = bd.sentencia(consulta)
        if respuesta.numeroResultados()!=0:
            accesos=[]
            tamano=[]
            tiempo=[]
            for indiceRespuesta in range(respuesta.numeroResultados()):
                dato=respuesta.entrada()
                accesos.append(dato[1])
                tamano.append(dato[2])
                tiempo.append(dato[3])
            fs=FuncionesEstadistica();
            valores=[accesos,tamano,tiempo]
            nombres=["Accesos","Tamaño","Tiempo"]
            i=0
            for valor in valores:
                resultado=[]
                resultado.append(nombres[i])
                i=i+1
                resultado.append(len(valor))
                resultado.append(fs.media(valor))
                resultado.append(fs.percentil(valor,50))
                resultado.append(fs.varianza(valor))
                resultado.append(min(valor))
                resultado.append(max(valor))
                resultado.append(fs.desviacion(valor))
                matriz.append(resultado)
        del respuesta
        bd.desconectar()
        return matriz

    def calcularResumenCSV(self):
        '''Calcula la matriz resumen de carga descriptiva por página dejándola en el archivo cuyo nombre se devuelve''' 
                       
        archivoCSV=""
        consulta="select cs_uri_sistema_llegada as archivo, count(cs_uri_sistema_llegada) as accesos,"
        consulta=consulta+" avg(sc_bytes) as tamano,"
        consulta=consulta+" avg(sc_time) as tiempo from hits"
        consulta=consulta+" where "+self.condicionEspecifica+" and "+self.condicionPaginas+" and "+self.condicionNoError
        if self.opcion<>0:
                consulta=consulta+" and "+self.condicionCeros
        consulta=consulta+" group by cs_uri_sistema_llegada;"
        bd=BaseDatosWeb()
        respuesta = bd.sentencia(consulta)
        if respuesta.numeroResultados()!=0:
            accesos=[]
            tamano=[]
            tiempo=[]
            archivoCSV=self.nombreArchivo()
            arch=MemoriaCSV(archivoCSV)
            for indiceRespuesta in range(respuesta.numeroResultados()):
                dato=respuesta.entrada()
                accesos.append(dato[1])
                tamano.append(dato[2])
                tiempo.append(dato[3])
                arch.crear()
                arch.escribir(dato)
            del arch
        del respuesta
        bd.desconectar()
        return archivoCSV


    def nombreArchivo(self):
        if self.tipo==0:
            tipo="desc"
        else:
            tipo="asc"
        if self.opcion==0:
            opcion="Accesos"
        elif self.opcion==1:
            opcion="Bytes"
        else:
            opcion="Tiempo"
        if self.ceros==0:
            ceros="de_ceros"
        elif self.ceros==1:
            ceros="con_ceros" 
        else:
            ceros="sin_ceros"
        return "P"+str(self.idproyecto)+"-Carga_Descriptiva_"+opcion+"_"+ceros+"_"+tipo+"-FecIni"+self.fechaIni+"-FecFin"+self.fechaFin+".csv"
    
        
''' ############################################################## '''
''' ###################  Paquete Estadisticas  ################### '''
''' ############################################################## '''

class MatrizTransiciones(object):
    '''
    Clase para representar las trenasiciones de los usuarios entre paginas
    '''
    
    def __init__(self):
        ''' Constructor '''
        self.salidas=[]
        self.llegadas=[]
        self.transiciones=[]
        self.numtransiciones=0;
    
    def getSalidas(self):
        return self.salidas
    
    def setSalidas(self, salida):
        self.salidas=salida
    
    def getLlegadas(self):
        return self.llegadas
    
    def setLlegadas(self, llegada):
        self.llegadas=llegada        
    
    def getTransicion(self, salida, llegada):
        resultado=(-1)
        if self.salidas.count(salida)>0:
            if self.llegadas.count(llegada)>0:
                indSal=self.salidas.index(salida)
                indLle=self.llegadas.index(llegada)
                resultado=self.transiciones[indSal][indLle]
        return resultado
    
    def setTransicion(self, salida, llegada, valor):
        if self.salidas.count(salida)==0:
            self.salidas.append(salida)
            ceros=[0]*(len(self.llegadas))
            self.transiciones.append(ceros[:])
        indSal=self.salidas.index(salida)
        if self.llegadas.count(llegada)==0:
            self.llegadas.append(llegada)
            for trans in self.transiciones:
                trans.append(0)
        indLle=self.llegadas.index(llegada)   
        self.transiciones[indSal][indLle]=valor
        if valor!=0.0:
            self.numtransiciones=self.numtransiciones+1
        else:
            self.numtransiciones=self.numtransiciones-1
                
    def getTransiciones(self):
        return self.transiciones[:]
    
    def setTransiciones(self, transicion):
        self.transiciones=transicion
    
    def getNumTransiciones(self):
        return self.numtransiciones
    
    
''' ######################################################## '''
''' ###################  Paquete Lector  ################### '''
''' ######################################################## '''

import math
import GeoIP as geo

class CreaLectorAbs(object):
    '''
    Clase para crear lectores de bitácoras 
    '''

    def crearlector(self, tipo):
        ''' Devuelve un lector del tipo indicado en el parámetro '''
        lector=Lector()
        return lector

        
class CreaLector(CreaLectorAbs):
    '''
    Clase para crear lectores de bitácoras según el tipo de archivo
    '''

    def crearlector(self, tipo):
        ''' Devuelve un lector del tipo indicado en el parámetro '''
        if tipo==4:
            lector=LectorIISMax()
        elif tipo==3:
            lector=LectorIISMin()
        elif tipo==1:
            lector=LectorApacheCommon()
        elif tipo==2:
            lector=LectorApacheCombined()
        else:
            lector=LectorApacheCommon()
        return lector


class Lector(object):
    '''
    Clase para leer bitácoras de tipo genérico
    '''
    
    def leer(self, archivo, idProyecto):
        bitac=Bitacora()
        numinsert=self.leerBitacora(archivo, idProyecto, bitac)
        del bitac
        return numinsert
    
    def leerBitacora(self, archivo, idProyecto, bitac):
        
        numinsert=0
        
        log=self.procesarLog(archivo,idProyecto)
            
        archivo.file.seek(0,2)
        tamano=archivo.file.tell()
        archivo.file.seek(0,0)
        actual=archivo.file.tell()
        memprog=MemoriaProgreso('DatosHAWUVA'+str(idProyecto)+'.txt')
        memprog.crear(tamano,0)
        memprog.setActual(actual)
            
        linea = archivo.file.readline()
        if linea:
            nuevolog=0
            nombrevis='MemInsVisHAWUVA'+str(idProyecto)+'.txt'
            memvis= MemoriaInsertar(nombrevis)
            memvis.crear()
            listVisit=ListaVisitas()
            numvis=0
            nombrehit='MemInsHitHAWUVA'+str(idProyecto)+'.txt'
            memhit= MemoriaInsertar(nombrehit)
            memhit.crear()
            numhits=0
            nombrevit='MemInsVitHAWUVA'+str(idProyecto)+'.txt'
            memvit= MemoriaInsertar(nombrevit)
            memvit.crear()
            mibasegeo = geo.open("/var/www/datos/GeoLiteCity.dat", geo.GEOIP_INDEX_CACHE | geo.GEOIP_CHECK_CACHE)
            vecesinsert=math.ceil(math.log10(tamano+2))
            porcen=math.ceil(100/vecesinsert)
            indicepor=1
            while linea:
                linea = linea.strip()
                if bitac.esBitacora(linea):
                    bitac.setBitacora(linea)
                    if nuevolog==0:
                        log.setInicio(bitac.getFechaHora())
                        nuevolog=1
                    numhits=numhits+1
                    if bitac.esPagina():
                        listCerrada=ListaVisitas()
                        hit=self.procesarPagina(listVisit, bitac, idProyecto, listCerrada)
                        numvis=numvis+listCerrada.escribir(memvis)
                        del listCerrada
                        visitante=self.procesarVisitante(bitac, idProyecto)
                        self.procesarLugarVisita(bitac, visitante, mibasegeo)
                        memvit.escribir(visitante.getDatos())
                        del visitante
                    else:
                        hit=self.procesarPeticion(listVisit, bitac, idProyecto)
                    memhit.escribir(hit.getDatos())
                    ultimafecha=hit.getFechaHora()
                    del hit
                actual=archivo.file.tell()
                if ((indicepor*porcen)>=100):
                    por=100
                else:
                    por=indicepor*porcen
                if ((actual*100)/tamano)>=por:
                    numinsertact=self.insertarObjetos(nombrehit, "hits")
                    numinsert=numinsert+numinsertact
                    if numinsertact>0:
                        self.insertarObjetos(nombrevit, "visitantes")
                    memhit.eliminar()
                    memvit.eliminar()
                    indicepor=indicepor+1
                    if ((actual*100)/tamano)<100:
                        memhit.crear()
                        memvit.crear()
                        self.insertarObjetos(nombrevis, "visitas")
                        memvis.eliminar()
                        memvis.crear()
                    elif numinsert>0:
                        numvis=numvis+listVisit.escribir(memvis)
                        self.insertarObjetos(nombrevis, "visitas")
                        memvis.eliminar()
                        log.setHits(numhits)
                        log.setFin(ultimafecha)
                        log.setVisitas(numvis)
                        controlLog=ControlLog()
                        controlLog.insertar(log)
                        del controlLog
                    else:
                        memvis.eliminar()
                memprog.setActual(actual)
                linea = archivo.file.readline()
            memprog.setActual(memprog.getMaximo())
            memprog.setTotal(numinsert)
            mibasegeo.close()
            del mibasegeo
            del listVisit
            del memhit
            del memvis
        del memprog
        del log
        return numinsert
    
    def leerFechas(self, archivo):
        bitac=Bitacora()
        fechas=self.leerFechasBitacora(archivo, bitac)
        del bitac
        return fechas
    
    def leerFechasBitacora(self, archivo, bitac):
        
        fechas=[]
        archivo.file.seek(0,0)
        linea = archivo.file.readline()
        if linea:
            ini=0
            while linea and ini==0:
                linea = linea.strip()
                if bitac.esBitacora(linea):
                    bitac.setBitacora(linea)
                    fechaIni=bitac.getFechaHora()
                    ini=1
                else:
                    linea = archivo.file.readline()
            fin=0
            if ini==1:
                posicion=0
                while fin==0:
                    posicion=posicion-1 
                    archivo.file.seek(posicion, 2)
                    actual=archivo.file.tell()
                    linea = archivo.file.readline()
                    linea = linea.strip()
                    if bitac.esBitacora(linea):
                        bitac.setBitacora(linea)
                        fechaFin=bitac.getFechaHora()
                        fin=1
        if ini==1 and fin==1:
            fechas.append(fechaIni)
            fechas.append(fechaFin)
        return fechas
    
    def procesarVisitante(self, bitac, idProyecto):
        raise NotImplementedError("El método no está implementado en esta clase")
    
    def procesarLugarVisita(self, bitac, visit, mibasegeo):
        ''' Añade el lugar de la visita al visitante pasado como parámetro '''
        '''if mibasegeo.record_by_name(bitac.getIp()):'''
        migeo=mibasegeo.record_by_name(bitac.getIp())
        if migeo!=None and len(migeo)>0:
            visit.setPais(migeo['country_name'])
            visit.setRegion(migeo['region_name'])
            visit.setCiudad(migeo['city'])
            visit.setLatitud(migeo['latitude'])
            visit.setLongitud(migeo['longitude'])
        
    
    def procesarPagina(self, listVisit, bitac, idProyecto, listCerrada):
        ''' Elimina todas las visitas cuyo ultimo hit fue hace mas de media hora de la lista de visitas pasada como parámetro listVisit. '''
        ''' Inserta una nueva visita en la lista de visitas con los datos de la bitacora pasada como parametro bitac o la modifica si esta ya existe. '''
        ''' Devuelve el nuevo hit generado por la linea de bitacora y la lista de visitas eliminadas añadidas al parametro listcerradas. '''
        nuevHora=datetime.strptime(bitac.getFechaHora(), '%Y-%m-%d %H:%M:%S')
        diferencia=timedelta(minutes=30)
        numvisita=-1
        indicevisita=0
        while indicevisita<listVisit.numero():
            visit=listVisit.getVisita(indicevisita)
            ultiHora=datetime.strptime(visit.getFin(), '%Y-%m-%d %H:%M:%S')
            if nuevHora>(ultiHora+diferencia):
                visit=listVisit.eliminar(indicevisita)
                listCerrada.insertar(visit)
            else:
                if visit.getIp()==bitac.getIp():
                    numvisita=indicevisita
                indicevisita=indicevisita+1
        if numvisita==-1:
            visit=Visita()
            visit.setProyecto(idProyecto)
            visit.setIp(bitac.getIp())
            visit.setInicio(bitac.getFechaHora())
            visit.setFin(bitac.getFechaHora())
            visit.setEntrada(bitac.getUri())
            visit.setSalida(bitac.getUri())
            listVisit.insertar(visit)
            salidaHit="?"
        else:
            visit=listVisit.getVisita(numvisita)
            entrada=visit.getEntrada()
            salidaHit=visit.getSalida()
            if not(self.esPagina(entrada)):
                visit.setEntrada(bitac.getUri())
                if not(self.esPagina(salidaHit)):
                    salidaHit="?"
            elif not(self.esPagina(salidaHit)):
                salidaHit=entrada
            visit.setSalida(bitac.getUri())
            visit.setFin(bitac.getFechaHora())
            listVisit.modificar(numvisita, visit)
        hit=bitac.toHit(idProyecto)
        hit.setSalida(salidaHit)
        return hit
    
    
    def procesarPeticion(self, listVisit, bitac, idProyecto):
        ''' Devuelve el nuevo hit generado por la linea de bitacora. '''
        numvisita=listVisit.buscar(bitac.getIp())
        if numvisita==-1:
            visit=Visita()
            visit.setProyecto(idProyecto)
            visit.setIp(bitac.getIp())
            visit.setInicio(bitac.getFechaHora())
            visit.setFin(bitac.getFechaHora())
            visit.setEntrada(bitac.getUri())
            visit.setSalida(bitac.getUri())
            listVisit.insertar(visit)
            salidaHit="?"
        else:
            visit=listVisit.getVisita(numvisita)
            entrada=visit.getEntrada()
            salidaHit=visit.getSalida()
            if not(self.esPagina(salidaHit)):
                if not(self.esPagina(entrada)):
                    salidaHit="?"
                else:
                    salidaHit=entrada
                visit.setSalida(bitac.getUri())
            elif not(self.esPagina(entrada)):
                visit.setEntrada(salidaHit)
                visit.setSalida(bitac.getUri())
            visit.setFin(bitac.getFechaHora())
            listVisit.modificar(numvisita, visit)
        hit=bitac.toHit(idProyecto)
        hit.setSalida(salidaHit)
        return hit
    
    
    def procesarLog(self, archivo, idProyecto):
        log=Log()
        log.setProyecto(idProyecto)
        nombre=os.path.basename(archivo.filename)
        log.setNombre(nombre)
        actualHora=datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        log.setFecha(actualHora)
        return log
    
    def esPagina(self, cadena):
        if re.match('(\S+).(htm|html|php|asp|psp|py)', cadena):
            resultado=True
        else:
            resultado=False
        return resultado
        
    def insertarObjetos(self, nombreArchivo, nombreTabla):
        consulta="LOAD DATA LOCAL INFILE '/var/www/tmp/"+nombreArchivo+"' IGNORE INTO TABLE "+nombreTabla+" FIELDS TERMINATED BY ',' LINES TERMINATED BY '\n'"
        bd=BaseDatosWeb()
        respuesta = bd.sentencia(consulta)
        resultado=respuesta.numeroResultados()
        bd.desconectar()
        return resultado
              

class LectorIISMax(Lector):
    '''
    Clase para leer bitácoras Microsoft IIS Log Extendido
    '''
    
    def leer(self, archivo, idProyecto):
        bitac=BitacoraIISMax()
        numinsert=self.leerBitacora(archivo, idProyecto, bitac)
        del bitac
        return numinsert
    
    def procesarVisitante(self, bitac, idProyecto):
        visit=Visitante()
        visit.setProyecto(idProyecto)
        visit.setIp(bitac.getIp())
        visit.setNombreUsuario(bitac.getNombreUsuario())
        return visit
    
    def leerFechas(self, archivo):
        bitac=BitacoraIISMax()
        fechas=self.leerFechasBitacora(archivo, bitac)
        del bitac
        return fechas


class LectorIISMin(Lector):
    '''
    Clase para leer bitácoras Microsoft IIS Log Minimo
    '''
    
    def leer(self, archivo, idProyecto):
        bitac=BitacoraIISMin()
        numinsert=self.leerBitacora(archivo, idProyecto, bitac)
        del bitac
        return numinsert
    
    def procesarVisitante(self, bitac, idProyecto):
        visit=Visitante()
        visit.setProyecto(idProyecto)
        visit.setIp(bitac.getIp())
        visit.setNombreUsuario(bitac.getNombreUsuario())
        return visit
    
    def leerFechas(self, archivo):
        bitac=BitacoraIISMin()
        fechas=self.leerFechasBitacora(archivo, bitac)
        del bitac
        return fechas
    
    
class LectorApacheCommon(Lector):
    '''
    Clase para leer bitácoras Apache Common Log
    '''
    
    def leer(self, archivo, idProyecto):
        bitac=BitacoraApacheCommon()
        numinsert=self.leerBitacora(archivo, idProyecto, bitac)
        del bitac
        return numinsert
    
    def procesarVisitante(self, bitac, idProyecto):
        visit=Visitante()
        visit.setProyecto(idProyecto)
        visit.setIp(bitac.getIp())
        visit.setIdUsuario(bitac.getIdUsuario())
        return visit
    
    def leerFechas(self, archivo):
        bitac=BitacoraApacheCommon()
        fechas=self.leerFechasBitacora(archivo, bitac)
        del bitac
        return fechas    


class LectorApacheCombined(Lector):
    '''
    Clase para leer bitácoras Apache Combined Log
    '''
    
    def leer(self, archivo, idProyecto):
        bitac=BitacoraApacheCombined()
        numinsert=self.leerBitacora(archivo, idProyecto, bitac)
        del bitac
        return numinsert
    
    def procesarVisitante(self, bitac, idProyecto):
        visit=Visitante()
        visit.setProyecto(idProyecto)
        visit.setIp(bitac.getIp())
        visit.setIdUsuario(bitac.getIdUsuario())
        return visit
    
    def leerFechas(self, archivo):
        bitac=BitacoraApacheCombined()
        fechas=self.leerFechasBitacora(archivo, bitac)
        del bitac
        return fechas    


''' ######################################################## '''
''' #################  Paquete Estadistica  ################ '''
''' ######################################################## '''
    
import numpy as np

class FuncionesEstadistica():
    '''
    Clase con las funciones estadísticas mas comunes.
    '''
    def media(self, a):
        a = np.asarray(a)
        a = a.astype(float)
        return np.mean(a,None)

    def percentil(self, a, per):
        values = np.sort(a,axis=0)
        idx = per /100. * (values.shape[0] - 1)
        values = values.astype(float)
        if (idx % 1 == 0):
            return values[idx]
        else:
            return (values[int(idx)]+values[int(idx) + 1])/2.

    def varianza(self, a):
        a = np.asarray(a)
        a = a.astype(float).ravel()
        n = len(a)
        return a.var()*(n/(n-1.))

    def desviacion(self, a):
        return np.sqrt(self.varianza(a))




