# !/usr/bin/env python
# -*- coding: utf-8 -*-
# @module persistencia.py
# @author: Gustavo

import MySQLdb as mdb
from config import *

''' ######################################################### '''
''' ################  Paquete Base de Datos ################# '''
''' ######################################################### '''

class BaseDatos(object):
    '''
    Clase abstracta para el acceso a base de datos.
    '''

    def conectar(self):
        ''' Conexi�n a la base de datos '''
        raise NotImplementedError
    
    def desconectar(self):
        ''' Desconexi�n de la base de datos '''
        raise NotImplementedError


class BaseDatosMySQL(BaseDatos):
    '''
    Clase para el acceso a base de datos MySQL
    '''
    
    enlace = None
    
    def __init__(self, host, usuario, contrasena, base):
        ''' Constructor '''
        self.conectar(host, usuario, contrasena, base)
        
    def conectar(self, host, usuario, contrasena, base):
        ''' Conexi�n a la base de datos MySQL '''
        self.enlace = mdb.connect(host, usuario, contrasena, base, local_infile = 1)
        
    def sentencia(self,consulta):
        ''' Lanza la consulta sobre la base de datos '''
        cursor = Consulta(consulta,self.enlace)
        return cursor        
        
    def desconectar(self):
        ''' Desconexi�n de la base de datos MySQL '''
        self.enlace.close()
        
    def __del__(self):
        self.desconectar()
        

class Consulta(object):
    '''
    Clase para la realizaci�n de una consulta a una base de datos MySQL
    '''
    
    cursor = None
            
    def __init__(self, consulta, base):
        ''' Constructor que ejecuta la consulta '''
        self.cursor = base.cursor()
        self.cursor.execute(consulta)
        base.commit()
           
    def numeroResultados(self):
        ''' N�mero de resultados de la consulta '''
        return self.cursor.rowcount
        
    def entrada(self):
        ''' Siguiente fila de la consulta '''
        return self.cursor.fetchone()
    
    def __del__(self):
        ''' Destructor que libera el cursor '''
        self.cursor.close()
        

class BaseDatosWeb(BaseDatosMySQL):
    '''
    Clase para el acceso a base de datos de la aplicaci�n.
    '''
    __host=HOST
    __usuario=USUARIO
    __contrasena=CONTRASENA
    __base=NOMBRE_BD
    
    enlace = None
    
    def __init__(self):
        ''' Constructor '''
        self.conectar(self.__host, self.__usuario, self.__contrasena, self.__base)
        
''' ######################################################### '''
''' ##################  Paquete Archivos  ################### '''
''' ######################################################### '''

import os
os.environ['MPLCONFIGDIR'] = RUTA_INSTALACION+"tmp/"
        
class MemoriaProgreso(object):
    
   
    def __init__(self, nombre):
        self.nombrefich=RUTA_INSTALACION+"tmp/"+nombre
    
    def crear(self, maximo, actual):
        fichcom = open(self.nombrefich, "w")
        fichcom.seek(0,0)
        fichcom.write(str(actual)+"\n"+str(maximo)+"\n"+str(-1)+"\n")
        fichcom.close()
        
    def existe(self):
        if open(self.nombrefich, "r"):
            retorno=True
        else:
            retorno=False
        return retorno
    
    def getMaximo(self):
        fichcom = open(self.nombrefich, "r")
        fichcom.seek(0,0)
        actu= int(fichcom.readline())
        maxi= int(fichcom.readline())
        fichcom.close()
        return maxi
    
    def getActual(self):
        fichcom = open(self.nombrefich, "r")
        fichcom.seek(0,0)
        actu= int(fichcom.readline())
        fichcom.close()
        return actu
    
    def setActual(self, actual):
        fichcom = open(self.nombrefich, "r+w")
        fichcom.seek(0,0)
        actu= int(fichcom.readline())
        maxi= int(fichcom.readline())
        tota= int(fichcom.readline())
        fichcom.seek(0,0)
        fichcom.write(str(actual)+"\n"+str(maxi)+"\n"+str(tota)+"\n")
        fichcom.close()
        
    def getTotal(self):
        fichcom = open(self.nombrefich, "r")
        fichcom.seek(0,0)
        actu= int(fichcom.readline())
        maxi= int(fichcom.readline())
        tota= int(fichcom.readline())
        fichcom.close()
        return tota
        
    def setTotal(self, total):
        fichcom = open(self.nombrefich, "r+w")
        fichcom.seek(0,0)
        actu= int(fichcom.readline())
        maxi= int(fichcom.readline())
        fichcom.write(str(total)+"\n")
        fichcom.close()
    
    def eliminar(self):
        os.remove(self.nombrefich)

        
class MemoriaDigrafo(object):
    
    def __init__(self, nombre):
        self.nombrefich=RUTA_INSTALACION+"tmp/"+nombre
      
    def crear(self):  
        fichcom = open(self.nombrefich, "w")
        fichcom.write('digraph g {')
        fichcom.write('\n')
        fichcom.close()
        
    def escribirNodo(self, nodo, caracteristicas):
        fichcom = open(self.nombrefich, "a")
        fichcom.write('    '+nodo+' ['+caracteristicas+']')
        fichcom.write('\n')
        fichcom.close()

    def escribirTransicion(self, origen, destino, peso):
        fichcom = open(self.nombrefich, "a")
        fichcom.write('    "'+origen+'" -> "'+destino+'" [label="'+str(peso)+'"]')
        fichcom.write('\n')
        fichcom.close()
    
    def cerrar(self):  
        fichcom = open(self.nombrefich, "a")
        fichcom.write('}')
        fichcom.close()
    
    def eliminar(self):
        os.remove(self.nombrefich)
        
        
class MemoriaCSV(object):
    
    def __init__(self, nombre):
        self.nombrefich=RUTA_INSTALACION+"tmp/"+nombre
      
    def crear(self):  
        fichcom = open(self.nombrefich, "w")
        fichcom.close()
        
    def escribir(self, vector):
        fichcom = open(self.nombrefich, "a")
        indice=0
        for dato in vector:
            if indice>0:
                fichcom.write(' ')
            else:
                indice=indice+1
            fichcom.write(str(dato))
        fichcom.write('\n')    
        fichcom.close()

    def eliminar(self):
        os.remove(self.nombrefich)


class MemoriaMatrizTransiciones(object):
    
        
    def __init__(self, nombre):
        self.nombrefich=RUTA_INSTALACION+"tmp/"+nombre

    def crear(self):  
        fichcom = open(self.nombrefich, "w")
        fichcom.close()
        
    def escribir(self, matriz):
        fichcom = open(self.nombrefich, "w")
        salidas=matriz.getSalidas()
        llegadas=matriz.getLlegadas()
        transiciones=matriz.getTransiciones()
        fichcom.seek(0,0)
        fichcom.write(str('Salidas\llegadas'))
        for llegada in llegadas:
            fichcom.write(' '+str(llegada))
        fichcom.write('\n')
        indSal=0
        for salida in salidas:
            fichcom.write(str(salida))
            datos=transiciones[indSal]
            for dato in datos:
                fichcom.write(' '+str(dato))
            fichcom.write('\n')      
            indSal=indSal+1   
        fichcom.close()
    
    def leer(self, matriz):
        fichcom = open(self.nombrefich, "r")
        fichcom.seek(0,0)
        linea=fichcom.readline()
        lleg=linea.split()
        leyenda=lleg.pop(0)
        matriz.setLlegadas(lleg)
        sali=[]
        tran=[]
        linea=fichcom.readline()
        while linea:
            tmp=linea.split()
            sali.append(tmp.pop(0))
            tmpint=[]
            for datotmp in tmp:
                tmpint.append(int(datotmp))
            tran.append(tmpint[:])
            linea=fichcom.readline()
        matriz.setTransiciones(tran[:])
        matriz.setSalidas(sali[:])
    
    def eliminar(self):
        os.remove(self.nombrefich)

class MemoriaInsertar(object):
    
    def __init__(self, nombre):
        self.nombrefich=RUTA_INSTALACION+"tmp/"+nombre
      
    def crear(self):  
        fichcom = open(self.nombrefich, "w")
        fichcom.close()

    def escribir(self, vector):
        fichcom = open(self.nombrefich, "a")
        for indicevalor in range(len(vector)):
            if (vector[indicevalor]=='-') or (vector[indicevalor]==''):
                fichcom.write('\\N')
            else:
                fichcom.write(str(vector[indicevalor]).replace("'","''"))
            if indicevalor<(len(vector)-1):
                fichcom.write(',')
        fichcom.write('\n')
        fichcom.close()
    
    def eliminar(self):
        os.remove(self.nombrefich)
